/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.basis.varianten;

import java.util.Collections;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Kulturkunde;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.zauber.Repraesentation;

public interface Varianten {

    /** Die Liste der Basis-Talente. */
    public static final Varianten BASIS_TALENTE =
        new VariantenFest( VariantenUtil.konvertieren( Talent.getBasisTalente() ) );

    /** Die Varianten fuer den Vorteil Begabung fuer Talent. */
    public static final Varianten BEGABUNG_TALENT =
        new VariantenBegabungTalent();

    /** Die Varianten fuer den Vorteil Besonderer Besitz */
    public static final Varianten BESONDERER_BESITZ =
        new VariantenBesondererBesitz();

    public static final Varianten DUMMY = new VariantenFest( Collections.EMPTY_LIST );

    /** Ein Liste von Giftarten. */
    public static final Varianten GIFTARTEN = new VariantenFrei(
            I18n.i18n( "Giftart.alchimistisch" ),
            I18n.i18n( "Giftart.Atemgifte" ),
            I18n.i18n( "Giftart.BlutWaffengifte" ),
            I18n.i18n( "Giftart.Einnahmegifte" ),
            I18n.i18n( "Giftart.Kontaktgifte" ),
            I18n.i18n( "Giftart.mineralisch" ),
            I18n.i18n( "Giftart.pflanzlisch" ),
            I18n.i18n( "Giftart.tierisch" ) );

    /** Ein Liste von Giften. */
    public static final Varianten GIFTE = new VariantenFrei(
            I18n.i18n( "Gift.Angstgift" ),
            I18n.i18n( "Gift.Arachnae" ),
            I18n.i18n( "Gift.Boabungaha" ),
            I18n.i18n( "Gift.Goldleim" ),
            I18n.i18n( "Gift.Gonede" ),
            I18n.i18n( "Gift.Halbgift" ),
            I18n.i18n( "Gift.Kelmon" ),
            I18n.i18n( "Gift.Kukris" ),
            I18n.i18n( "Gift.Purpurblitz" ),
            I18n.i18n( "Gift.Wurara" ) );

    /**
     * Fuer Sonderfertigkeiten die keine Varianten zur Verfuegung stellen.
     */
    public static final Varianten KEINE = new VariantenKeine();

    /**
     * Die Varianten fuer die Sonderfertigkeit Kulturkunde.
     */
    public static final Varianten KULTURKUNDE =
        new VariantenFest( VariantenUtil.konvertieren( Kulturkunde.getKulturkunden() ) );

    /** Ein Liste von Krankheiten. */
    public static final Varianten KRANKHEITEN = new VariantenFrei(
            I18n.i18n( "Krankheit.Triefnase" ),
            I18n.i18n( "Krankheit.FlinkerDifar" ),
            I18n.i18n( "Krankheit.BlutigerRotz" ),
            I18n.i18n( "Krankheit.Dumpfschaedel" ),
            I18n.i18n( "Krankheit.Wundfieber" ),
            I18n.i18n( "Krankheit.Paraylse" ),
            I18n.i18n( "Krankheit.Tollwut" ) );

    /**
     * Die moeglichen Talente fuer den Vorteil Meisterhandwerk.
     */
    public static final Varianten MEISTERHANDWERK =
        new VariantenMeisterhandwerk();

    /** Die Varianten fuer die Sonderfertigkeit Repraesentation. */
    public static final Varianten REPRAESENTATION =
        new VariantenFest( VariantenUtil.konvertieren( Repraesentation.getRepraesentationen() ) );

    /** Die Varianten fuer die Sonderfertigkeit Ruestungsgewoehnung. */
    public static final Varianten RUESTUNGEN = new VariantenRuestungen();

    /**
     * Die moeglichen Talentgruppen fuer Begabung/Unfaehigkeit fuer
     * [Talentgruppe].
     */
    public static final Varianten TALENTGRUPPE =
        new VariantenTalentgruppe();

    /**
     * Die Varianten fuer den Vorteil Uebernatuerliche Begabung.
     */
    public static final Varianten UEBERNATUERLICHE_BEGABUNG =
        new VariantenUebernatuerlicheBegabung();

    /**
     * Fuegt der Liste der Varianten eine neue Variante hinzu.
     * Die Methode macht nichts, wenn die Variante schon in der Liste ist.
     *
     * @param variante die in die Liste der Varianten eingefuegt werden soll.
     */
    public void einfuegen( final Variante variante );

    /**
     * Liefert die korrekte Variante zu einem gewaehlten Objekt. Das Objekt kann
     * entweder schon die Variante sein, oder es handelt sich um eine freie
     * Variante, d.h. einen vom Benutzer eingegebenen String. Ist es ein String,
     * so wird die dem String entsprechende Variante zurueckgeliefert. Evtl. 
     * wird eine neue Variante erzeugt.
     *  
     * @param objekt vom Type Variante oder String.
     * @return Die passende Variante.
     */
    public Variante getVariante( final Object objekt );
    
    /**
     * @param held
     *            fuer den die Varianten sein sollen. Es kann hier {@code null}
     *            uebergeben werden. In einem solchen Fall werden dann aber
     *            evtl. keine Varianten geliefert.
     * @return eine Liste aller Varianten. Die Liste kann auch leer sein wenn
     *         alle Varianten vom Benutzer eingegeben werden muessen.
     */
    public List< Variante > getVarianten(Held held);

    /**
     * @return {@code true}, wenn die Sonderfertigkeit/der Vor-/Nachteil
     *         Varianten besitzt, sonst {@code false}.
     */
    public boolean hatVarianten();

    /**
     * Freie Varianten bedeutet, dass die Variante vom Benutzer eingegeben
     * werden kann.
     *
     * @return {@code true}, wenn es sich um eine freie Variante handelt, sonst
     *      {@code false}.
     */
    public boolean istFreieVariante();
}
