/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.bausteine.bonus;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_BESCHREIBUNG;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_VARIANTE_AENDERBAR;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_WERTE;

import java.util.ArrayList;
import java.util.List;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.util.Strings;

public class SonderfertigkeitBonusStandard implements SonderfertigkeitBonus {

    /** Die Id, die in der XML-Datei fuer die Beschreibung verwendet wurde. */
    private final String beschreibungsId;

    /** Gib an ob die Variante der Werte geaendert werde darf. */
    private final boolean varianteAenderbar;

    /** Die Werte der Liste. */
    private final List< SonderfertigkeitWertBonus > werte;

    /**
     * @param dieBeschreibungsId
     *            Eine Id eines Textes aus der Lokalisierungsdatei oder die
     *            Beschreibung selbst. Der Text ist eine Beschreibung der Liste
     *            fuer den Benutzer. Wird von der {@link #toString()}-Methode
     *            zurueck geliefert. Wenn {@code null} uebergeben wird liefert
     *            {@code toString()} eine Aufzaehlung der Werte mit oder, siehe
     *            {@link Strings#erzeugeAufzaehlungMitOder(List)}.
     * @param istVarianteAenderbar
     *            gibt an ob die Variante der Sonderfertigkeiten geaendert
     *            werden darf.
     * @param dieWerte
     *            aus denen man bei dem Bonus waehlen darf.
     */
    public SonderfertigkeitBonusStandard(
            final String dieBeschreibungsId,
            final boolean istVarianteAenderbar,
            final List< SonderfertigkeitWertBonus > dieWerte ) {

        werte = new ArrayList< SonderfertigkeitWertBonus >( dieWerte );

        beschreibungsId = dieBeschreibungsId;

        varianteAenderbar = istVarianteAenderbar;
    }

    /**
     * {@inheritDoc}
     */
    public boolean enthaelt( final Sonderfertigkeit sonderfertigkeit ) {

        for ( final SonderfertigkeitWertBonus bonus : werte ) {

            if ( bonus.enthaelt( sonderfertigkeit ) ) {

                return true;
            }
        }

        return false;
    }

    /**
     * @return die GP-Kosten, die einem Baustein, durch diesen Bonus entstehen.
     */
    public final double getGpKosten() {

        double kosten = 0.0;

        for ( final SonderfertigkeitWertBonus bonus : werte ) {

            kosten += bonus.getWert().getApKosten();
        }

        kosten /= werte.size();

        kosten /= 100.0;

        return kosten;
    }

    /** {@inheritDoc} */
    public final List<SonderfertigkeitWert> getWerte( final Held held ) {

        final List< SonderfertigkeitWert > kopie =
            new ArrayList< SonderfertigkeitWert >( werte.size() );

        for ( final SonderfertigkeitWertBonus wert : werte ) {

            kopie.add( wert.getWert() );
        }

        return kopie;
    }

    /**
     * @return eine Beschreibung des Bonus fuer den Benutzer.
     */
    @Override
    public String toString() {

        if ( beschreibungsId == null ) {

            return Strings.erzeugeAufzaehlungMitOder( werte );

        } else if ( I18n.hatLokalisierungstext( beschreibungsId ) ) {

            return I18n.i18n( beschreibungsId );

        } else {

            return beschreibungsId;
        }
    }

    public final IXMLElement toXml() {

        final IXMLElement element = new XMLElement( TAG_WERTE );

        if ( null != beschreibungsId ) {

            element.setAttribute( ATTR_BESCHREIBUNG, beschreibungsId );
        }

        if ( varianteAenderbar ) {

            element.setAttribute( ATTR_VARIANTE_AENDERBAR, "true" );
        }

        for ( final SonderfertigkeitWertBonus wert : werte ) {

            element.addChild( wert.toXml() );
        }

        return element;
    }
}
