/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.daten;

import static de.bernhardjung.dsaprogramm.daten.Datenkonstanten.DATEIERWEITERUNG_KULTUR;
import static de.bernhardjung.dsaprogramm.daten.Datenkonstanten.DATEIERWEITERUNG_PROFESSION;
import static de.bernhardjung.dsaprogramm.daten.Datenkonstanten.DATEIERWEITERUNG_RASSE;
import static de.bernhardjung.dsaprogramm.daten.Datenkonstanten.DATEI_FERNKAMPFWAFFEN;
import static de.bernhardjung.dsaprogramm.daten.Datenkonstanten.DATEI_NAHKAMPFWAFFEN;
import static de.bernhardjung.dsaprogramm.daten.Datenkonstanten.DATEI_NAMENLISTEN;
import static de.bernhardjung.dsaprogramm.daten.Datenkonstanten.DATEI_NAME_FACTORYS;
import static de.bernhardjung.dsaprogramm.daten.Datenkonstanten.DATEI_RUESTUNGEN;
import static de.bernhardjung.dsaprogramm.daten.Datenkonstanten.DATEI_SCHILDE;
import static de.bernhardjung.dsaprogramm.daten.Datenkonstanten.VERZEICHNIS_KULTUREN;
import static de.bernhardjung.dsaprogramm.daten.Datenkonstanten.VERZEICHNIS_PROFESSIONEN;
import static de.bernhardjung.dsaprogramm.daten.Datenkonstanten.VERZEICHNIS_RASSEN;
import static de.bernhardjung.dsaprogramm.daten.xml.FernwaffeXmlTags.TAG_FERNWAFFEN;
import static de.bernhardjung.dsaprogramm.daten.xml.NahkampfwaffeXmlTags.TAG_WAFFEN;
import static de.bernhardjung.dsaprogramm.daten.xml.NamenFactoryXmlTags.TAG_NAME_FACTORYS;
import static de.bernhardjung.dsaprogramm.daten.xml.NamenlisteXmlTags.TAG_NAMENLISTEN;
import static de.bernhardjung.dsaprogramm.daten.xml.RuestungXmlTags.TAG_RUESTUNGEN;
import static de.bernhardjung.dsaprogramm.daten.xml.SchildXmlTags.TAG_SCHILDE;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Collection;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import net.n3.nanoxml.XMLWriter;
import de.bernhardjung.dsaprogramm.basis.Fernkampfwaffe;
import de.bernhardjung.dsaprogramm.basis.Nahkampfwaffe;
import de.bernhardjung.dsaprogramm.basis.Ruestung;
import de.bernhardjung.dsaprogramm.basis.Schild;
import de.bernhardjung.dsaprogramm.basis.namen.NameFactory;
import de.bernhardjung.dsaprogramm.basis.namen.Namenliste;
import de.bernhardjung.dsaprogramm.bausteine.Heldenbaustein;
import de.bernhardjung.dsaprogramm.gui.DialogFehlermeldung;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Einstellungen;

public class Datenspeicherer {

    private static String ermitteleDateiname( final String id ) {

        final String idDateiname = id + ".Dateiname";

        final String name;

        if ( I18n.hatLokalisierungstext( idDateiname ) ) {

            name = I18n.i18n( idDateiname );

        } else {

            final int index = id.indexOf( "." );

            name = id.substring( 1 + index );
        }

        return name;
    }

    /**
     * Speicher alle Daten in ein angegebenes Verzeichnis.
     *
     * @param verzeichnis in das gespeichert werden soll.
     */
    public static void speichereDaten( final File verzeichnis ) {

        final Datenspeicher speicher = Datenspeicher.getInstanz();

        // Falls das Verzeichnis noch nicht existiert wird es angelegt.
        // Falls es schon existiert geschieht hier nichts.
        verzeichnis.mkdir();

        // Jetzt wird geprueft ob die benoetigten Unterverzeichnisse
        // existieren. Verzeichnisse, die noch nicht existieren werden
        // angelegt und gefuellt.

        // Fernwaffen
        // **********
        final File dateiFernkampfwaffen =
            new File( verzeichnis, DATEI_FERNKAMPFWAFFEN );

        if ( ! dateiFernkampfwaffen.exists() ) {

            speichereFernkampfwaffen();
        }

        // Kulturen
        // ********
        final File verzeichnisKulturen =
            new File( verzeichnis, VERZEICHNIS_KULTUREN );

        if ( ! verzeichnisKulturen.exists() ) {

            speichernHeldenbaustein(
                    speicher.getKulturen(),
                    verzeichnisKulturen,
                    DATEIERWEITERUNG_KULTUR );
        }

        // Nahkampfwaffen
        // **************
        final File dateiNahkampfwaffen =
            new File( verzeichnis, DATEI_NAHKAMPFWAFFEN );

        if ( ! dateiNahkampfwaffen.exists() ) {

            speichereNahkampfwaffen();
        }
        
        // NameFactorys
        // *************
        final File dateiNameFactorys =
            new File( verzeichnis, DATEI_NAME_FACTORYS );

        if ( ! dateiNameFactorys.exists() ) {

            speichereNameFactorys();
        }

        // NameFactorys
        // *************
        final File dateiNamelisten =
            new File( verzeichnis, DATEI_NAMENLISTEN );

        if ( ! dateiNamelisten.exists() ) {

            speichereNamenlisten();
        }

        // Professionen
        // ************
        final File verzeichnisProfessionen =
            new File( verzeichnis, VERZEICHNIS_PROFESSIONEN );

        if ( ! verzeichnisProfessionen.exists() ) {

            speichernHeldenbaustein(
                    speicher.getProfessionen(),
                    verzeichnisProfessionen,
                    DATEIERWEITERUNG_PROFESSION );
        }
        
        // Rassen
        // ******
        final File verzeichnisRassen =
            new File( verzeichnis, VERZEICHNIS_RASSEN );

        if ( ! verzeichnisRassen.exists() ) {

            speichernHeldenbaustein(
                    speicher.getRassen(),
                    verzeichnisRassen,
                    DATEIERWEITERUNG_RASSE );
        }

        // Ruestungen
        // **********
        final File dateiRuestungen =
            new File( verzeichnis, DATEI_RUESTUNGEN );

        if ( ! dateiRuestungen.exists() ) {

            speichereRuestungen();
        }

        // Schilde
        // *******.
        final File dateiSchilde =
            new File( verzeichnis, DATEI_SCHILDE );

        if ( ! dateiSchilde.exists() ) {

            speichereSchilde();
        }
    }

    /**
     * Speichert die Fernwaffen in das Verzeichnis, das als Datenquelle
     * eingestellt ist.
     */
    public static void speichereFernkampfwaffen() {

        final IXMLElement wurzelElement = new XMLElement( TAG_FERNWAFFEN );

        final Datenspeicher speicher = Datenspeicher.getInstanz();

        for ( final Fernkampfwaffe waffe : speicher.getFernkampfwaffen() ) {

            wurzelElement.addChild( waffe.toXml() );
        }

        final String basisverzeichnis = Einstellungen.getInstanz().getDatenquelle();

        final File datei = new File( basisverzeichnis, DATEI_FERNKAMPFWAFFEN );

        speichern( wurzelElement, datei );
    }

    /**
     * Speichert die Nahkampfwaffen in das Verzeichnis, das als Datenquelle
     * eingestellt ist.
     */
    public static void speichereNahkampfwaffen() {

        final IXMLElement wurzelElement = new XMLElement( TAG_WAFFEN );

        final Datenspeicher speicher = Datenspeicher.getInstanz();

        for ( final Nahkampfwaffe waffe : speicher.getNahkampfwaffen() ) {

            wurzelElement.addChild( waffe.toXml() );
        }

        final String basisverzeichnis = Einstellungen.getInstanz().getDatenquelle();

        final File datei = new File( basisverzeichnis, DATEI_NAHKAMPFWAFFEN );

        speichern( wurzelElement, datei );
    }

    /**
     * Speichert die NameFactorys in das Verzeichnis, das als Datenquelle
     * eingestellt ist.
     */
    public static void speichereNameFactorys() {

        final IXMLElement wurzelElement = new XMLElement( TAG_NAME_FACTORYS );

        final Datenspeicher speicher = Datenspeicher.getInstanz();

        for ( final NameFactory factory : speicher.getNameFactorys() ) {

            wurzelElement.addChild( factory.toXml() );
        }

        final String basisverzeichnis = Einstellungen.getInstanz().getDatenquelle();

        final File datei = new File( basisverzeichnis, DATEI_NAME_FACTORYS );

        speichern( wurzelElement, datei );
    }

    /**
     * Speichert die Namenlisten in das Verzeichnis, das als Datenquelle
     * eingestellt ist.
     */
    public static void speichereNamenlisten() {

        final IXMLElement wurzelElement = new XMLElement( TAG_NAMENLISTEN );

        final Datenspeicher speicher = Datenspeicher.getInstanz();

        for ( final Namenliste liste : speicher.getNamenlisten() ) {

            wurzelElement.addChild( liste.toXml() );
        }

        final String basisverzeichnis = Einstellungen.getInstanz().getDatenquelle();

        final File datei = new File( basisverzeichnis, DATEI_NAMENLISTEN );

        speichern( wurzelElement, datei );
    }

    /**
     * Speichert die Ruestungen in das Verzeichnis, das als Datenquelle
     * eingestellt ist.
     */
    public static void speichereRuestungen() {

        final IXMLElement wurzelElement = new XMLElement( TAG_RUESTUNGEN );

        final Datenspeicher speicher = Datenspeicher.getInstanz();

        for ( final Ruestung ruestung : speicher.getRuestungen() ) {

            wurzelElement.addChild( ruestung.toXml() );
        }

        final String basisverzeichnis = Einstellungen.getInstanz().getDatenquelle();

        final File datei = new File( basisverzeichnis, DATEI_RUESTUNGEN );

        speichern( wurzelElement, datei );
    }

    /**
     * Speichert die Schild in das Verzeichnis, das als Datenquelle
     * eingestellt ist.
     */
    public static void speichereSchilde() {

        final IXMLElement wurzelElement = new XMLElement( TAG_SCHILDE );

        final Datenspeicher speicher = Datenspeicher.getInstanz();

        for ( final Schild schild : speicher.getSchilde() ) {

            wurzelElement.addChild( schild.toXml() );
        }

        final String basisverzeichnis = Einstellungen.getInstanz().getDatenquelle();

        final File datei = new File( basisverzeichnis, DATEI_SCHILDE );

        speichern( wurzelElement, datei );
    }

    private static void speichern( final IXMLElement wurzelElement, final File datei ) {

        try {

            final OutputStream out = new BufferedOutputStream(
                    new FileOutputStream( datei ) );

            final XMLWriter writer = new XMLWriter( out );

            writer.write( wurzelElement, true );

        } catch ( final FileNotFoundException e ) {

            DialogFehlermeldung.zeigeDialog( String.format(
                    "Die Datei %1$s kann nicht angelegt werden.",
                    datei.getName() ), e );

        } catch ( final IOException e ) {

            DialogFehlermeldung.zeigeDialog( String.format(
                    "In die Datei %1$s kann nicht geschrieben werden.",
                    datei.getName() ), e );
        }
    }

    static < T extends Heldenbaustein > void speichernHeldenbaustein(
            final Collection< T > bausteine,
            final File verzeichnis,
            final String dateiendung ) {

        verzeichnis.mkdir();

        for ( final Heldenbaustein baustein : bausteine ) {

            final String dateiname = ermitteleDateiname( baustein.getId())
                                   + dateiendung;

            speichern( baustein.toXml(), new File( verzeichnis,
                                                   dateiname ) );
        }
    }
}
