/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.daten.factorys.bausteine;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_STUFE;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_VARIANTE;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_VARIANTE_AENDERBAR;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_VORNACHTEIL;

import java.util.ArrayList;
import java.util.List;

import net.n3.nanoxml.IXMLElement;
import de.bernhardjung.dsaprogramm.bausteine.bonus.VorNachteilBonus;
import de.bernhardjung.dsaprogramm.bausteine.bonus.VorNachteilWertBonus;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

public class FactoryVorNachteileIn {

    /**
     * Das XML-Element, das eine Liste von Daten enthaelt um daraus
     * VorNachteilBonus-Objekte zu erzeugen.
     */
    private final String xmlElement;

    /**
     * @param dasXmlElement
     *            das eine Liste von Daten enthaelt um daraus
     *            VorNachteilBonus-Objekte zu erzeugen.
     */
    public FactoryVorNachteileIn( final String dasXmlElement ) {
        xmlElement = dasXmlElement;
    }

    /**
     * Ermittelt die Variante aus einem Wert-Element des XML-Dokuments.
     *
     * @param wertElement
     *            das auf eine Variante hin untersucht werden soll.
     * @return eine Variante oder {@code null} wenn das Element keine Variante
     *         vorgibt.
     */
    private Object ermittleVariante( final IXMLElement wertElement ) {

        final String varianteId =
            wertElement.getAttribute( ATTR_VARIANTE );

        if ( null == varianteId ) {
            return null;
        }

        if ( varianteId.startsWith( "Talentgruppe." ) ) {

            return Talentgruppe.getTalentgruppe( varianteId );

        } else if ( varianteId.startsWith( "Talent." ) ) {

            return Talent.getTalent( varianteId );

        } else if ( I18n.hatLokalisierungstext( varianteId ) ) {

            return I18n.i18n( varianteId );

        } else  {

            return varianteId;

        }
    }

    private VorNachteilBonus erzeugeBonus( final IXMLElement werteElement ) {

        final List< VorNachteilWertBonus > boni =new ArrayList< VorNachteilWertBonus >();

        // Fuer den Fall dass die Werte Varianten haben muss noch
        // festgestellt werden ob die Variante vom Benutzer geaendert
        // werden darf
        final boolean varianteAenderbar;

        final String varianteAenderbarString =
            werteElement.getAttribute( ATTR_VARIANTE_AENDERBAR );

        if ( null == varianteAenderbarString ) {

            varianteAenderbar = false;

        } else {

            varianteAenderbar = Boolean.valueOf( varianteAenderbarString );

        }

        // Jetzt die einzelnen Werte erzeugen.
        for ( final IXMLElement wertElement : werteElement.getChildren() ) {

            // Erst den Vor-/Nachteil ermitteln.
            final String id = wertElement.getAttribute( ATTR_VORNACHTEIL );

            final VorNachteil vorNachteil = VorNachteil.getVorNachteil( id );

            // Variante ermitteln.
            final Object variante = ermittleVariante( wertElement );

            final VorNachteilWert wert;

            final VorNachteilWertBonus bonus;

            if ( null == variante ) {

                wert = new VorNachteilWert( vorNachteil );

                bonus = new VorNachteilWertBonus( wert );

            } else {

                wert = new VorNachteilWert( vorNachteil,
                                            variante,
                                            varianteAenderbar );

                bonus = new VorNachteilWertBonus(
                        wert,
                        wertElement.getAttribute( ATTR_VARIANTE ) );
            }

            // Jetzt noch pruefen ob fuer den Wert eine Stufe vorgegeben ist.
            final String stufeString =
                wertElement.getAttribute( ATTR_STUFE );

            if ( null != stufeString ) {

                final int stufe = Integer.valueOf( stufeString );

                wert.setStufe( stufe );
            }

            boni.add( bonus );
        }

        return new VorNachteilBonus( boni, varianteAenderbar );
    }

    /**
     * Erzeugt aus dem XML-Dokument die Vor-/Nachteil die ein Held durch
     * den Baustein erhaelt.
     *
     * @param wurzelElement
     *            des XML-Dokuments.
     * @return die aus dem XML-Dokument ermittelten Vor-/Nachteile.
     */
    public final List< VorNachteilBonus > erzeugen(
            final IXMLElement wurzelElement ) {

        final List< VorNachteilBonus > boni =
            new ArrayList< VorNachteilBonus >();

        final IXMLElement boniElement =
            wurzelElement.getFirstChildNamed( xmlElement );

        if ( null != boniElement ) {

            for ( final IXMLElement bonus : boniElement.getChildren() ) {

                boni.add( erzeugeBonus( bonus ) );
            }
        }

        return boni;
    }
}
