/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import javax.swing.AbstractListModel;
import javax.swing.MutableComboBoxModel;

import de.bernhardjung.dsaprogramm.util.Uebersetzbar;

public class CharGenListModel< S, T extends S >
extends AbstractListModel< S >
implements MutableComboBoxModel< S >, Uebersetzbar {

    /** Zum Sortieren der Elemente. */
    private final Comparator< S > comparator;
    
    /** Das aktuell in der Combobox angezeigte Element. */
    private T selectedItem = null;

    /** Die Elemente die die Comobbox zur Auswahl stellt. */
    private final List< T > elemente = new ArrayList< T >();

    public CharGenListModel() {
        
        comparator = null;
    }

    public CharGenListModel( final Comparator< S > derComparator ) {
        
        comparator = derComparator;
    }

    public CharGenListModel( final Collection< T > dieElemente ) {
        
        comparator = null;
        
        setElements( dieElemente );
    }
    
    public CharGenListModel(
            final Collection< T > dieElemente,
            final Comparator< S > derComparator ) {
        
        comparator = derComparator;
        
        setElements( dieElemente );
    }

    /** {@inheritDoc} */
    @SuppressWarnings("unchecked")
    public final void addElement(final Object obj) {

        final T element = (T) obj;
        
        int index; 
        
        if ( null == comparator ) {
        
            index = Math.max( 0, getSize() - 1 );
            
        } else {
            
            index = Collections.binarySearch( 
                    elemente, 
                    element, 
                    comparator );
            
            if ( 0 <= index ) {
                
                return;
            }

            index = - ( index + 1 );
        }
        
        elemente.add( index, element );

        fireIntervalAdded( this, index, index );
    }

    /** {@inheritDoc} */
    public final Object getSelectedItem() {
        return selectedItem;
    }

    /** {@inheritDoc} */
    @SuppressWarnings("unchecked")
    public final void setSelectedItem(final Object obj ) {

        final T anItem = (T) obj;
        
        if ( ( null != anItem ) ) {

            if( ! anItem.equals( selectedItem ) ) {

                selectedItem = anItem;

                fireContentsChanged( this, -1, -1 );
            }

        } else if ( null != selectedItem ) {

            selectedItem = null;

            fireContentsChanged( this, -1, -1 );
        }
    }
    
    /**
     * Leert den Inhalt des Models.
     */
    public final void clear() {
        
        final int lastIndex = getSize() - 1;
        
        elemente.clear();
        
        fireContentsChanged( this, 0, lastIndex );
    }
    
    /**
     * Prueft ob das uebergebene Elemente in der Liste ist.
     * 
     * @param obj
     * @return {@code true}, wenn das Element in der Liste ist, sonst 
     *          {@code false}.
     */
    @SuppressWarnings("unchecked")
    public final boolean contains( final Object obj ) {
        
        if ( null == obj ) {
            
            return false;
        
        } else if ( null == comparator ) {
            
            return elemente.contains( obj );
        
        } else {
            
            return 0 <= Collections.binarySearch( 
                    elemente, 
                    (T) obj,
                    comparator );
        }
    }
    
    /** {@inheritDoc} */
    public final T getElementAt(final int index) {
        
        return elemente.get( index );
    }

    /**
     * @return Die Elemente aus dem Model.
     */
    public List< T > getElements() {
        
        return new ArrayList< T >( elemente );
    }

    /**
     * @return Das gewaehlte Element oder {@code null}, wenn kein Element
     *          gewaehlt ist.
     */
    public final T getGewaehltesElement() {
        
        return selectedItem;
    }
    
    /** {@inheritDoc} */
    public final int getSize() {
        
        return elemente.size();
    }

    /** 
     * 
     */
    @SuppressWarnings("unchecked")
    public void insertElementAt( final Object obj, final int index ) {
        
        if ( null == comparator )
        {
            elemente.add( index, (T) obj );
    
            fireIntervalAdded( this, index, index );
        
        } else {
            
            addElement( obj );
        }
    }
    
    /**
     * @return {@code true}, wenn das Model keine Elemente enthaelt, sonst
     *         {@code false}.
     */
    public final boolean isEmpty() {
        
        return 0 == elemente.size();
    }
    
    public final void neuUebersetzen() {
        
        if ( isEmpty() ) {
            
            return;
        }
        
        if ( null != comparator ) {
            
            Collections.sort( elemente, comparator );
        }
        
        final int letzerIndex = getSize() - 1;
        
        fireContentsChanged( this, 0, letzerIndex );
        
        fireContentsChanged( this, -1, -1 );
    }

    /** {@inheritDoc} */
    public void removeElement( final Object obj ) {

        final int index = elemente.indexOf( obj );

        if ( index >= 0 ) {

            removeElementAt( index );
        }
    }

    /** {@inheritDoc} */
    public void removeElementAt( final int index ) {

        if ( getElementAt( index ) == selectedItem ) {
            if ( index == 0 ) {
                setSelectedItem( getSize() == 1 ? null : getElementAt( index + 1 ) );
            }
            else {
                setSelectedItem( getElementAt( index - 1 ) );
            }
        }

        elemente.remove( index );

        fireIntervalRemoved( this, index, index );
    }

    /**
     * Entfernt das gewaehlte Element aus der Liste. Macht nichts, wenn kein
     * Element gewaehlt ist.
     */
    public final void removeSelectedElement() {
        
        if ( null == selectedItem ) {
            
            return;
        }
        
        removeElement( selectedItem );        
    }

    /**
     * Leert das Model und fuellt es mit dem uebergebenen Inhalt neu.
     * 
     * @param neueElemente Die neuen Varianten fuer die ComboBox.
     */
    public final void setElements( final Collection< T > neueElemente ) {

        final int alteGroesse = getSize();
        
        elemente.clear();

        elemente.addAll( neueElemente );
        
        final int neueGroesse = getSize();
        
        if ( alteGroesse < neueGroesse ) {
            
            fireIntervalAdded( this, alteGroesse, neueGroesse - 1 );
        
        } else if ( alteGroesse > neueGroesse ) {
            
            fireIntervalRemoved( this, neueGroesse, alteGroesse - 1 );
        }
        
        if ( null != comparator ) {
            
            Collections.sort( elemente, comparator );
        }
        
        fireContentsChanged( this, 0, getSize() - 1 );

        if ( contains( selectedItem ) ) {
            
            return;
        
        } else if ( 0 < elemente.size() ) {

            setSelectedItem( elemente.get( 0 ) );

        } else {

            setSelectedItem( null );
        }
    }
    
    /**
     * Leert das Model und fuellt es mit dem uebergebenen Inhalt neu.
     * 
     * @param neueElemente Die neuen Varianten fuer die ComboBox.
     */
    public final void setElements( 
            final Collection< T > neueElemente,
            final T gewaehltesElement ) {

        final int alteGroesse = getSize();
        
        elemente.clear();

        elemente.addAll( neueElemente );
        
        final int neueGroesse = getSize();
        
        if ( alteGroesse < neueGroesse ) {
            
            fireIntervalAdded( this, alteGroesse, neueGroesse - 1 );
        
        } else if ( alteGroesse > neueGroesse ) {
            
            fireIntervalRemoved( this, neueGroesse, alteGroesse - 1 );
        }
        
        if ( null != comparator ) {
            
            Collections.sort( elemente, comparator );
        }
        
        fireContentsChanged( this, 0, getSize() - 1 );

        setSelectedItem( gewaehltesElement );
    }
}
