/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui;

import static de.bernhardjung.dsaprogramm.i18n.I18n.i18n;

import java.awt.Font;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JLabel;
import javax.swing.JPanel;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.erschaffung.ErschaffungManager;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Util;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

public class PanelGp extends JPanel implements Observer, Uebersetzbar {

    /**
     * Der Tooltip, wenn das AP-Konto ueberzogen wurde.
     */
    private static final String AP_UEBERZOGEN = "PanelProbleme.ApUeberzogen";

    /**
     * Ein String in dem mittels {@link String#format(String, Object[])}
     * die Gp Punkte eingetragen werden.
     * Zum Beispiel: "%1$s GP zu verteilen".
     */
    private static final String GP = "PanelGp.GpZurVerfuegung";

    /** Der Tooltip, wenn das GP-Konto ueberzogen wurde. */
    private static final String GP_UEBERZOGEN = "PanelProbleme.GpUeberzogen";

    /**
     * Ein String in dem mittels {@link String#format(String, Object[])}
     * die Ap Punkte eingetragen werden.
     * Zum Beispiel: "%1$s AP zu verteilen".
     */
    private static final String AP = "PanelGp.ApZurVerfuegung";

    /**
     * Ein String der auch die AP anzeigt, die noch durch den Vorteil Gebildet
     * zu verteilen sind.
     * Zum Beispiel: "%1$s (%2$s) AP zu verteilen".
     */
    private static final String AP_GEBILDET =
        "PanelGp.ApZurVerfuegungGebildet";

    /**
     * Ein String der auch die AP anzeigt, die noch durch den Vorteil Gebildet
     * zu verteilen sind und es werden die AP angezeigt, die noch auf Zauber
     * verteilt werden duerfen.
     * Zum Beispiel: "%1$s (%2$s) AP zu verteilen, davon %3$s AP fuer Zauber".
     */
    private static final String AP_GEBILDET_ZAUBER =
        "PanelGp.ApZurVerfuegungGebildetZauber";

    /**
     * Ein String der die AP anzeigt, die noch auf Zauber verteilt werden
     * duerfen.
     * Zum Beispiel: "%1$s AP zu verteilen. %3$s AP davon fuer Zauber".
     */
    private static final String AP_ZAUBER =
        "PanelGp.ApZurVerfuegungZauber";

    /** Das Label das die GP anzeigt. */
    private final JLabel labelGp;

    /** Das Label das die AP anzeigt. */
    private final JLabel labelAp;

    /**
     * Der ErschaffungManager von dem das Panel den aktuellen Stand von
     * AP und GP erfaehrt.
     */
    private ErschaffungManager manager;

    PanelGp() {

        manager = new ErschaffungManager();
        
        final Font schrift = getFont();
        final int schriftgroesse = schrift.getSize();
        final float neueSchriftgroesse = schriftgroesse * 1.5F;
        final Font grosseSchrift =
            schrift.deriveFont( Font.BOLD, neueSchriftgroesse );

        labelGp = new JLabel( "110 GP" );
        labelGp.setFont( grosseSchrift );

        labelAp = new JLabel( "500 AP" );
        labelAp.setFont( grosseSchrift );

        setLayout( new FormLayout( "7dlu, F:P:G, P, 7dlu",
                                   "7dlu, P, 7dlu" ) );

        final CellConstraints cc = new CellConstraints();

        add( labelGp, cc.xy( 2, 2 ) );
        add( labelAp, cc.xy( 3, 2 ) );
    }

    public void neuUebersetzen() {
        
        update( null, null );
    }
    
    /**
     * Aendert den {@link ErschaffungManager} wenn ein neuer Held erschaffen
     * wird. Ueber den Manager erfaehrt das Panel wie viele GP und AP noch zu
     * verteilen sind.
     */
    public final void setManager(
            final ErschaffungManager neuerManager ) {

        manager = neuerManager;

        manager.addObserver( this );

        manager.getHeld().addObserver( this );

        update( null, null );
    }

    /**
     * Informiert das Panel darueber das sich die Werte der angezeigten GP und
     * AP geaendert haben und aktualisiert die Anzeige.
     *
     * @param o ungenutzt.
     * @param arg ungenutzt.
     */
    public final void update( final Observable o, final Object arg ) {
        
        final int restGp = manager.getGpGuthaben();

        if ( restGp < 0 ) {

            labelGp.setForeground( Util.FEHLERFARBE );
            labelGp.setToolTipText( I18n.i18n( GP_UEBERZOGEN ) );

        } else {

            labelGp.setForeground( getForeground() );
            labelGp.setToolTipText( null );

        }
        labelGp.setText( String.format( i18n( GP ), restGp ) );

        final int restAp = manager.getApGuthaben();

        final int restApZauber = manager.getApGuthabenMagie();

        if ( ( restAp < 0 ) || ( restApZauber < 0 ) ) {

            labelAp.setForeground( Util.FEHLERFARBE );
            labelAp.setToolTipText( I18n.i18n( AP_UEBERZOGEN ) );

        } else {

            labelAp.setForeground( getForeground() );
            labelAp.setToolTipText( null );

        }

        final Held held = manager.getHeld();

        final boolean gebildet = held.hat( VorNachteil.GEBILDET );

        final boolean zauberer = held.istZauberer();

        if ( zauberer && gebildet ) {

            labelAp.setText( String.format(
                    i18n( AP_GEBILDET_ZAUBER ),
                    restAp,
                    manager.getApGuthabenGebildet(),
                    manager.getApGuthabenMagie() ) );

        } else if ( zauberer ) {

            labelAp.setText( String.format(
                    i18n( AP_ZAUBER ),
                    restAp,
                    manager.getApGuthabenMagie() ) );

        } else if ( gebildet ) {

            labelAp.setText( String.format(
                    i18n( AP_GEBILDET ),
                    restAp,
                    manager.getApGuthabenGebildet() ) );

        } else {

            labelAp.setText( String.format( i18n( AP ), restAp ) );
        }
    }
}
