/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui;

import java.util.ArrayList;
import java.util.List;

import de.bernhardjung.dsaprogramm.basiswerte.Basiswert;
import de.bernhardjung.dsaprogramm.bausteine.Kultur;
import de.bernhardjung.dsaprogramm.bausteine.Rasse;
import de.bernhardjung.dsaprogramm.bausteine.bonus.Talentbonus;
import de.bernhardjung.dsaprogramm.bausteine.bonus.VorNachteilBonus;
import de.bernhardjung.dsaprogramm.bausteine.ungeeignete.UngeeigneterVorNachteil;
import de.bernhardjung.dsaprogramm.bausteine.util.EigenschaftsModifikation;
import de.bernhardjung.dsaprogramm.bausteine.voraussetzungen.Voraussetzung;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.util.Strings;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

class PanelRasse extends PanelHeldenbaustein< Rasse > {

    /** Die Einheit fuer die AE-Modifikation, z.B. AsP. */
    private static final String AE_EINHEIT =
        "PanelRasse.AeModEinheit";

    /** Text bei der AE-Modifikation. */
    private static final String AE_MOD = "PanelRasse.AeMod";

    /** Text bei der Formel fuer das Alter. */
    private static final String ALTER = "PanelRasse.Alter";

    /** Die Einheit fuer die AU-Modifikation, z.B. AuP. */
    private static final String AU_EINHEIT =
        "PanelRasse.AuModEinheit";

    /** Text bei der AU-Modifikation. */
    private static final String AU_MOD = "PanelRasse.AuMod";

    /** Ueberschrift fuer die Tabelle mit den Augenfarben. */
    private static final String AUGENFARBEN =
        "PanelRasse.Augenfarben";

    /** Text bei den Eigenschafts-Modifikationen. */
    private static final String EIGENSCHAFTS_MODIFIKATIONEN =
        "PanelRasse.EigenschaftsModifikationen";

    /** Text mit dem die Formel fuer das Gewicht eines Helden angezeigt wird.*/
    private static final String GEWICHT = "PanelRasse.Gewicht";

    /** Ueberschrift fuer die Tabelle mit den Haarfarben. */
    private static final String HAARFARBEN =
        "PanelRasse.Haarfarben";

    /** Die Einheit fuer das Alter, d.h. "Jahre". */
    private static final String JAHRE = "PanelRasse.Jahre";

    /** Zeigt an, dass die Rasse keine Talent-Boni vergibt. */
    private static final String KEINE_MODIFIKATIONEN =
        "PanelRasse.KeineModifikationen";

    /** Text mit dem die Koerpergroesse Formel angezeigt wird. */
    private static final String KOERPERGROESSE =
        "PanelRasse.Koerpergroesse";

    /** Text bei der Liste mit den Kulturen. */
    private static final String KULTUREN =
        "PanelRasse.Kulturen";

    /** Text bei der LE-Modifikation. */
    private static final String LE_MOD = "PanelRasse.LeMod";

    /** Die Einheit fuer die LE-Modifikation, z.B. LeP. */
    private static final String LE_EINHEIT =
        "PanelRasse.LeModEinheit";

    /** Text bei der MR-Modifikation. */
    private static final String MR_MOD = "PanelRasse.MrMod";

    /** Das Panel mit die Kultur des Helden gewaehlt wird. */
    private final PanelKultur panelKultur;

    /**
     * Erzeugt das Panel.
     *
     * @param rassen die zur Auswahl stehen.
     * @param dasPanelKultur
     */
    PanelRasse( final PanelKultur dasPanelKultur ) {

        super();

        panelKultur = dasPanelKultur;
    }

    /** {@inheritDoc} */
    @Override
    boolean bausteineGeaendert( final Object arg ) {
        return Datenspeicher.RASSEN.equals( arg );
    }

    /** {@inheritDoc} */
    @Override
    Rasse[] getBausteine() {

        final List< Rasse > rassen = Datenspeicher.getInstanz().getRassen();

        return rassen.toArray( new Rasse[ rassen.size() ] );
    }

    /** {@inheritDoc} */
    @Override
    final String getInformationen( final Rasse rasse ) {

        final StringBuilder informationen =
            new StringBuilder( "<html><body>" );

        informationen.append( "<p><b>" );
        informationen.append( rasse.getName() );
        informationen.append( "</b></p>");

        informationen.append( "<p><b>" );
        informationen.append( I18n.i18n( GENERIERUNGSKOSTEN ) );
        informationen.append( ": " );
        informationen.append( rasse.getGpKosten( held ) );
        informationen.append( " " );
        informationen.append( I18n.i18n( GP ) );
        informationen.append( "</b></p>" );

        informationen.append( "<p><b>" );
        informationen.append( I18n.i18n( ALTER ) );
        informationen.append( ": </b>" );
        informationen.append( rasse.getAlter().toString() );
        informationen.append( " " );
        informationen.append( I18n.i18n( JAHRE ) );
        informationen.append( "</p>" );

        informationen.append( "<p><b>" );
        informationen.append( I18n.i18n( HAARFARBEN ) );
        informationen.append( ": </b>" );
        informationen.append( rasse.getHaarfarben().toHtmlTabelleHorizontal());
        informationen.append( "</p>" );

        informationen.append( "<p><b>" );
        informationen.append( I18n.i18n( AUGENFARBEN ) );
        informationen.append( ": </b>" );
        informationen.append(rasse.getAugenfarben().toHtmlTabelleHorizontal());
        informationen.append( "</p>" );

        informationen.append( "<p><b>" );
        informationen.append( I18n.i18n( KOERPERGROESSE ) );
        informationen.append( ": </b>" );
        informationen.append( rasse.getGroesse().toString() );
        informationen.append( "</p>" );

        informationen.append( "<p><b>" );
        informationen.append( I18n.i18n( GEWICHT ) );
        informationen.append( ": </b>" );
        informationen.append( rasse.getGewicht().toString() );
        informationen.append( "</p>" );

        // *****************************
        // Voraussetzungen
        // *****************************
        final List< Voraussetzung > voraussetzungen;
        voraussetzungen = rasse.getVoraussetzungen();

        if ( 0 < voraussetzungen.size() ) {

            informationen.append( "<p><b>" );
            informationen.append( I18n.i18n( VORAUSSETZUNGEN ) );
            informationen.append( ":</b> " );

            final String text;
            if ( 0 == voraussetzungen.size() ) {

                text = "-";

            } else {

                text = Strings.erzeugeAufzaehlung( rasse.getVoraussetzungen());

            }

            informationen.append( text );
            informationen.append( "</p>" );
        }

        // *****************************
        // Modifikationen
        // *****************************
        informationen.append( "<p><b>" );
        informationen.append( I18n.i18n( EIGENSCHAFTS_MODIFIKATIONEN ) );
        informationen.append( ":</b> " );

        final List< EigenschaftsModifikation > modifikationen =
        	rasse.getEigenschaftsModifikationen();

        if ( ! modifikationen.isEmpty() ) {

            informationen.append(
                    Strings.erzeugeAufzaehlung( modifikationen ) );

        } else {

            informationen.append( I18n.i18n( KEINE ) );

        }
        informationen.append( "</p>" );

        informationen.append( "<p><b>" );
        informationen.append( I18n.i18n( LE_MOD ) );
        informationen.append( ":</b> +" );
        informationen.append( rasse.getModifikation( Basiswert.LE ) );
        informationen.append( " " );
        informationen.append( I18n.i18n( LE_EINHEIT ) );

        informationen.append( "&nbsp;&nbsp;&nbsp;<b>" );
        informationen.append( I18n.i18n( AU_MOD ) );
        informationen.append( ":</b> +" );
        informationen.append( rasse.getModifikation( Basiswert.AU ) );
        informationen.append( " " );
        informationen.append( I18n.i18n( AU_EINHEIT ) );

        int aeMod = 0;
        int mrMod = 0;

        if ( vergibtVollzauberer( rasse ) ) {

            aeMod = 12;

            mrMod = 2;

        } else  if ( vergibtViertelzauberer( rasse ) ) {

            aeMod = -6;

        }

        if ( rasse.hatModifikation( Basiswert.AE ) ) {

            aeMod += rasse.getModifikation( Basiswert.AE );
        }

        if ( 0 != aeMod ) {
            informationen.append( "</p><p><b>" );
            informationen.append( I18n.i18n( AE_MOD ) );
            informationen.append(
                    String.format( ":</b> %1$+d", aeMod ) );
            informationen.append( " " );
            informationen.append( I18n.i18n( AE_EINHEIT ) );
        }

        informationen.append( "&nbsp;&nbsp;&nbsp;<b>" );
        informationen.append( I18n.i18n( MR_MOD ) );
        informationen.append( ":</b> " );
        informationen.append( mrMod + rasse.getModifikation( Basiswert.MR ) );
        informationen.append( "</p>" );

        // *****************************
        // Automatische Vor-/Nachteile
        // *****************************
        informationen.append( "<p><b>" );
        informationen.append( I18n.i18n( AUTOMATISCHE_VOR_NACHTEILE ) );
        informationen.append( ": </b>" );

        final List< VorNachteilBonus > vorteilBoni = rasse.getVorteile();

        if ( 0 < vorteilBoni.size() ) {

            informationen.append( Strings.erzeugeAufzaehlung( vorteilBoni ) );
        }

        final int verbindungen = rasse.getVerbindungen();

        if ( 0 != verbindungen ) {

            if ( 0 < vorteilBoni.size() ) {

                informationen.append( ", " );
            }

            informationen.append( String.format( 
                    I18n.i18n( VERBINDUNGEN ),
                    verbindungen ) );
        }

        final List< VorNachteilBonus > nachteilBoni = rasse.getNachteile();

        if ( 0 < nachteilBoni.size() ) {

            if ( ( 0 < vorteilBoni.size() ) || ( 0 != verbindungen ) ) {

                informationen.append( " / " );
            }

            informationen.append( Strings.erzeugeAufzaehlung( nachteilBoni ) );
        }

        if ( ( 0 == vorteilBoni.size() ) && ( 0 == nachteilBoni.size() ) ) {

            informationen.append( I18n.i18n( KEINE ) );
        }

        informationen.append( "</p>" );

        // *************************
        // Empfohlene Vor-/Nachteile
        // *************************
        informationen.append( "<p><b>" );
        informationen.append( I18n.i18n( EMPFOHLENE_VOR_NACHTEILE ) );
        informationen.append( ": </b>" );

        final List< VorNachteil > empfohleneVorteile =
            rasse.getEmpfohleneVorteile();

        if ( 0 < empfohleneVorteile.size() ) {

            informationen.append(
                    Strings.erzeugeAufzaehlung( empfohleneVorteile ) );
        }

        final List< VorNachteil > empfohleneNachteile =
            rasse.getEmpfohleneNachteile();

        if ( 0 < empfohleneNachteile.size() ) {

            if ( 0 < empfohleneVorteile.size() ) {

                informationen.append( " / " );
            }

            informationen.append(
                    Strings.erzeugeAufzaehlung( empfohleneNachteile ) );
        }

        if ( ( 0 == empfohleneVorteile.size() )
          && ( 0 == empfohleneNachteile.size() ) ) {

            informationen.append( I18n.i18n( KEINE_BESTIMMTEN ) );
        }

        informationen.append( "</p>" );

        // **************************
        // Ungeeignete Vor-/Nachteile
        // **************************
        informationen.append( "<p><b>" );
        informationen.append( I18n.i18n( UNGEEIGNETE_VOR_NACHTEILE ) );
        informationen.append( ": </b>" );

        final List< UngeeigneterVorNachteil > ungeeigneteVorteile =
            rasse.getUngeeigneteVorteile();

        if ( 0 < ungeeigneteVorteile.size() ) {

            informationen.append(
                    Strings.erzeugeAufzaehlung( ungeeigneteVorteile ) );
        }

        final List< UngeeigneterVorNachteil > ungeeigneteNachteile =
            rasse.getUngeeigneteNachteile();

        if ( 0 < ungeeigneteNachteile.size() ) {

            if ( 0 < ungeeigneteVorteile.size() ) {

                informationen.append( " / " );
            }

            informationen.append(
                    Strings.erzeugeAufzaehlung( ungeeigneteNachteile ) );
        }

        if ( ( 0 == ungeeigneteVorteile.size() )
          && ( 0 == ungeeigneteNachteile.size() ) ) {

            informationen.append( I18n.i18n( KEINE_BESTIMMTEN ) );
        }

        informationen.append( "</p>" );

        // *****************************
        // Kulturen
        // *****************************
        informationen.append( "<p><b>" );
        informationen.append( I18n.i18n( KULTUREN ) );
        informationen.append( ": </b>" );
        informationen.append( Strings.erzeugeAufzaehlung(
                new ArrayList< Kultur >( rasse.getKulturen() ) ) );
        informationen.append( "</p>" );

        // *****************************
        // Talente
        // *****************************
        informationen.append( "<p><b>" );
        informationen.append( I18n.i18n( TALENTE ) );
        informationen.append( ": </b>" );

        // gibt an ob die Rasse Talent-Boni vergibt.
        boolean talentmodifikationen = false;

        for (final Talentgruppe talentgruppe: Talentgruppe.getTalentgruppen()){

            final List< Talentbonus > boni =
                rasse.getTalentboni( talentgruppe );

            if ( 0 < boni.size() ) {
                // Die Rasse vergibt in dieser Talentgruppe Boni.

                informationen.append( "<i>" );
                informationen.append( talentgruppe.getKuerzel() );
                informationen.append( ":</i> " );

                if ( 0 < boni.size() ) {

                    final String aufzaehlung;
                    aufzaehlung = Strings.erzeugeAufzaehlung( boni );

                    informationen.append( aufzaehlung );
                }

                informationen.append( "; " );

                talentmodifikationen = true;
            }
        }

        if ( ! talentmodifikationen ) {
            informationen.append( I18n.i18n( KEINE_MODIFIKATIONEN ) );
        }
        informationen.append( "</p>" );

        if ( rasse.hatLeittalente() ) {

            informationen.append( "<p><b>" );
            informationen.append( I18n.i18n( LEITTALENTE ) );
            informationen.append( ":</b> " );

            final String aufzaehlung;
            aufzaehlung = Strings.erzeugeAufzaehlung( rasse.getLeittalente() );
            informationen.append( aufzaehlung );
            informationen.append( "</p>" );
        }

        informationen.append( "</body></html>");

        return informationen.toString();
    }

    /**
     * Aendert die Rasse des Helden der gerade erschaffen wird.
     *
     * @param neueRasse des Helden.
     */
    @Override
    final void setGewaehlterBaustein( final Rasse neueRasse ) {

        if ( null != neueRasse ) {

            held.setRasse( neueRasse );

            panelKultur.setMoeglicheKulturen( neueRasse.getKulturen() );

        } else {

            held.setRasse( Rasse.LEERE_RASSE );
            panelKultur.setMoeglicheKulturen( Rasse.LEERE_RASSE.getKulturen() );
        }
    }
}
