/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basis.varianten.Variante;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.*;
import java.util.List;

class PanelVorNachteileListe
extends ResetbaresPanel 
implements Observer, Uebersetzbar {

    /**
     * Der Tooltiptext fuer das Icon mit dem empfohlene Vor-/Nachteile
     * gekennzeichnet sind.
     */
    private static final String EMPFOHLEN =
        "PanelVorNachteileListe.empfohlen";

    /** Ein String um die GP-Kosten anzuzeigen, z.B. "%1$s GP" */
    private static final String GP_KOSTEN =
        PanelSonderfertigkeitenListe.GP_KOSTEN;

    /** Das Icon mit dem ein empfohlener Vor-/Nachteil markiert wird. */
    private static final ImageIcon ICON_EMPFOHLEN =
        Util.ladeIcon( "empfohlen" );
    /**
     * Ein leeres Icon das als Platzhalter eingefuegt wird, wenn ein
     * Vor-/Nachteil weder empfohlen noch ungeeignet ist.
     */
    private static final ImageIcon ICON_PLATZHALTER =
        Util.ladeIcon( "platzhalter" );

    /** Das Icon mit dem ein ungeeigneter Vor-/Nachteil markiert wird. */
    private static final ImageIcon ICON_UNGEEIGNET =
        Util.ladeIcon( "ungeeignet" );

    /** Die Ueberschrift fuer die Spalte mit den Kosten. */
    private static final String KOSTEN =
        "PanelVorNachteileListe.Kosten";

    /**
     * Der Tooltiptext fuer das Icon mit dem ungeeignete Vor-/Nachteile
     * gekennzeichnet sind.
     */
    private static final String UNGEEIGNET =
        "PanelVorNachteileListe.ungeeignet";

    /**
     * Die Map speichert Regelelemente (Eigenschaften, Talente usw.) als
     * Schluessel.
     * Die Listen enthalten Indizes von Vor-/Nachteilen aus {@link #werte}.
     * Die Werte in den Listen sind von den entsprechenden Vor-/Nachteilen
     * abhaengig.
     */
    private final Map< Object, List< Integer > > abhaengigkeiten;

    /**
     * Die Action die aufgerufen wird, wenn der Benutzer eine Checkbox anklickt
     * um einen Vor-/Nachteil zu (de)aktivieren.
     */
    private final ActionListener actionDeAktivieren;

    /** Wird aufgerufen wenn die Variante eines Wertes gewaehlt wird. */
    private final ActionListener actionVariante;

    /**
     * Wenn im Panel die Werte eines Helden angezeigt werden wird in dieser
     * Variable gemerkt wie viele Vor-/Nachteile das Panel zeigt, die der
     * Held auch besitzt. Wenn sich diese Anzahl aendert, muss das Panel neu
     * aufgebaut werden.
     */
    private int anzahlGewaehlte;

    /**
     * Eine fette (bold) Schrift zum Markieren von vorgegebenen Vor-/Nachteilen.
     */
    private final Font fetteSchrift;

    /**
     * Enthaelt die Label die die GP-Kosten der Vor-/Nachteile anzeigen.
     * Das Label an Position i zeigt die Kosten des entsprechenden Wertes in
     * {@link #werte} an.
     */
    private final List< JLabel > gpKosten;

    /** Der Held fuer den die Vor-/Nachteile angezeigt werden. */
    private Held held;

    /**
     * Eine Map in der jeder Spinner unter dem index des Vor-/Nachteils
     * gespeichert ist auf den er sich bezieht.
     */
    private final Map< Integer, JSpinner > indexSpinner;

    /** Das Panel auf dem die Liste der Vor- und Nachteile angezeigt wird. */
    private final JPanel inhalt;

    /**
     * Die Uberschrift fuer die Spalte in der die GP Kosten der Vor-/Nachteile 
     * angzeigt werden.
     */
    private final JLabel kostenUeberschrift;
    
    /**
     * Die Markierungen die angeben ob ein Vor-/Nachteil empfohlen oder
     * ungeeignet ist.
     */
    private final List< JLabel > markierungen;

    
    
    /** Der Listener der auf aenderungen an der Stufe eines Wertes reagiert. */
    private final ChangeListener stufeListener;

    /**
     * Die Ueberschrift fuer die Spalte in der die Namen der Vor-/Nachteile 
     * angezeigt werden.
     */
    private final JLabel spaltenUeberschrift;
    
    /*  TODO *
     * Ein Panel auf dem die Punkte fuer Verbindungen verteilt werden koennen,
     * die man durch Rasse, Kultur und Profession erhalten hat.
     * /
    private PanelVerbindungenListe panelVerbindungen;
*/
    /** Uebersetzt die Benutzeroberflaeche. */
    private final Uebersetzer uebersetzer;
    
    /**
     * Enthaelt die Comboboxen zum auswaehlen der Varianten. Jede Combobox ist
     * unter dem Index des Wertes in {@link #werte} den sie editiert abgelegt.
     */
    private final Map< Integer, JComboBox > varianten;

    /**
     * Die Checkboxen mit denen ein Vor-/Nachteil (de)aktiviert werden kann. Die
     * Checkbox an Position i steht fuer den entsprechenden Wert in
     * {@link #werte}.
     */
    private final List< JCheckBox > vorNachteile;

    /** Die Werte die im Panel dargestellt werden. */
    private final List< VorNachteilWert > werte;

    /** Gib an ob das Panel die Werte eines Helden anzeigt. */
    private final boolean zeigtHeldenwerte;

    PanelVorNachteileListe(
            final boolean zeigtHeldenwerte,
            final String derSpaltenueberschriftL10nSchluessel ) {

        uebersetzer = new Uebersetzer();
        
        abhaengigkeiten = new HashMap<Object, List<Integer>>();

        actionDeAktivieren = new ActionDeAktivieren();

//        if ( zeigtHeldenwerte ) {

//            actionVariante = new ActionVarianteHeldenwerte();

//        } else {

            actionVariante = new ActionVariante();

//        }

        anzahlGewaehlte = 0;

        fetteSchrift = getFont().deriveFont( Font.BOLD );

        gpKosten = new ArrayList< JLabel >();

        indexSpinner = new HashMap< Integer, JSpinner >();

        inhalt = new JPanel();

        kostenUeberschrift = uebersetzer.erzeugeUeberschrift( KOSTEN );

        markierungen = new ArrayList< JLabel >();

        stufeListener = new StufeListener();

        spaltenUeberschrift = uebersetzer.erzeugeUeberschrift( 
                derSpaltenueberschriftL10nSchluessel );
        
        vorNachteile = new ArrayList< JCheckBox >();

        varianten = new HashMap< Integer, JComboBox >();

        werte = new ArrayList< VorNachteilWert >();

        this.zeigtHeldenwerte = zeigtHeldenwerte;

        final JScrollPane scrollPane = Util.erzeugeScrollPane( inhalt );
        resetter.einfuegen( scrollPane );
        
        setLayout( new BorderLayout() );

        add( scrollPane );
    }

    private void aufbauenBenutzeroberflaeche() {

        // falls die Benutzeroberflaeche neue aufgebaut wird muessen erst
        // alle alten Elemente der Oberflaeche entfernt werden.
        inhalt.removeAll();

        gpKosten.clear();
        indexSpinner.clear();
        markierungen.clear();
        vorNachteile.clear();
        varianten.clear();

        // Das neue Layout festlegen.
        final GroupLayout layout = new GroupLayout( inhalt );

        layout.setAutoCreateContainerGaps( true );

        layout.setAutoCreateGaps( true );

        inhalt.setLayout( layout );

        final GroupLayout.ParallelGroup hGroup = layout.createParallelGroup();

        layout.setHorizontalGroup( hGroup );

        final GroupLayout.SequentialGroup vGroup = layout.createSequentialGroup();

        layout.setVerticalGroup( vGroup );

        GroupLayout.SequentialGroup zeileH = layout.createSequentialGroup();
        GroupLayout.ParallelGroup zeileV;
        zeileV = layout.createParallelGroup( GroupLayout.Alignment.BASELINE );

        zeileH.addComponent( spaltenUeberschrift,
                    GroupLayout.PREFERRED_SIZE,
                    GroupLayout.PREFERRED_SIZE,
                    Short.MAX_VALUE );
        zeileV.addComponent( spaltenUeberschrift );

        zeileH.addComponent( kostenUeberschrift,
                    55,
                    55,
                    GroupLayout.PREFERRED_SIZE );
        zeileV.addComponent( kostenUeberschrift );

        hGroup.addGroup( zeileH );
        vGroup.addGroup( zeileV );

        // Alle 5 Zeilen wird ein Trennstrich eingefuegt, das erleichtert dem
        // Benutzer die Orientierung in den Zeilen der Liste.
        final int extraZeilen = 1 + ( werte.size() / 4 );

        // Jetzt die Liste Zeile fuer Zeile aufbauen.
        for ( int i = 0, n = werte.size() + extraZeilen, trennstriche = 0
                ; i < n
                ; ++i ) {

            zeileH = layout.createSequentialGroup();
            zeileV = layout.createParallelGroup( GroupLayout.Alignment.CENTER );

            if ( ( i % 5 ) == 0 ) {
                // ein Trennstrich wird eingefuegt.

                final JSeparator separator = new JSeparator();

                zeileH.addComponent( separator );
                zeileV.addComponent( separator,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE );

                hGroup.addGroup( zeileH );
                vGroup.addGroup( zeileV );

                ++trennstriche;

                continue;
            }

            // Das ist der index des Wertes in werte, der in der aktuellen
            // Zeile dargestellt wird.
            final int index = i - trennstriche;

            final String indexString = String.valueOf( index );

            final VorNachteilWert wert = werte.get( index );

            final VorNachteil vorNachteil = wert.getVorNachteil();

            // Die Markierung ob der Vor-/Nachteil empfohlen, ungeeignet oder
            // keines von beidem ist.
            final JLabel markierung = new JLabel();

            markierungen.add( markierung );

            zeileH.addComponent( markierung );
            zeileV.addComponent( markierung,
                    GroupLayout.PREFERRED_SIZE,
                    GroupLayout.PREFERRED_SIZE,
                    GroupLayout.PREFERRED_SIZE );

            // Spalte mit der Checkbox und evtl mit der Combobox aufbauen.
            if ( wert.istVarianteAenderbar() ) {

                // Hat der Wert eine Variante wird neben der Checkbox
                // eine Combobox benoetigt um die Variante waehlen zu koennen.

                final JCheckBox checkBox = new JCheckBox();

                checkBox.setText( vorNachteil.getPraefix() );

                checkBox.setName( indexString );

                checkBox.addActionListener( actionDeAktivieren );

                vorNachteile.add( checkBox );

                zeileH.addComponent( checkBox );
                zeileV.addComponent( checkBox,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE );

                final CharGenListModel< Variante, Variante > model =
                    new CharGenListModel< Variante, Variante >(
                            Variante.NAME_COMPARATOR );
                
                final JComboBox< Variante > comboBox = new JComboBox< Variante >( model );

                comboBox.addActionListener( actionVariante );

                comboBox.setEditable( vorNachteil.hatFreieVarianten() );

                varianten.put( index, comboBox );
                comboBox.setName( indexString );

                zeileH.addComponent( comboBox, 100, 100, Short.MAX_VALUE );
                zeileV.addComponent( comboBox,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE );

            } else {

                final JCheckBox checkBox = new JCheckBox( wert.getName() );

                checkBox.setName( indexString );

                checkBox.addActionListener( actionDeAktivieren );

                vorNachteile.add( checkBox );

                zeileH.addComponent( checkBox,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        Short.MAX_VALUE );
                zeileV.addComponent( checkBox,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE );

            }

            if ( vorNachteil.hatStufen( wert ) ) {

                final int minimum = vorNachteil.getMindeststufe( wert );

                final int schrittweite = vorNachteil.getSchrittweite( wert );

                final JSpinner spinner =
                    Util.erzeugeSpinner( minimum, 0, 99, schrittweite );

                spinner.addChangeListener( stufeListener );

                spinner.setName( String.valueOf( index ) );
                indexSpinner.put( index, spinner );

                zeileH.addComponent( spinner,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE );
                zeileV.addComponent( spinner,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE );
            }

            final JLabel gp = new JLabel( "12 GP" );

            gpKosten.add( gp );

            zeileH.addComponent( gp, 55, 55, GroupLayout.PREFERRED_SIZE );
            zeileV.addComponent( gp,
                    GroupLayout.PREFERRED_SIZE,
                    GroupLayout.PREFERRED_SIZE,
                    GroupLayout.PREFERRED_SIZE );

            hGroup.addGroup( zeileH );
            vGroup.addGroup( zeileV );
        }

        if ( 0 < werte.size() ) {

            layout.linkSize(
                    SwingConstants.HORIZONTAL,
                    gpKosten.toArray( new Component[ gpKosten.size() ] ) );

        }
    }

    /**
     * Initialisiert {@link #abhaengigkeiten} neu.
     */
    private void initAbhaengigkeiten() {

        abhaengigkeiten.clear();

        for ( int i = 0, n = werte.size(); i < n ; ++i ) {

            final VorNachteil vorNachteil = werte.get( i ).getVorNachteil();

            for ( final Object regelelement : vorNachteil.getAbhaengigkeiten() ) {

                setAbhaengigkeit( regelelement, i );
            }
        }
    }
    
    public void neuUebersetzen() {

        uebersetzer.neuUebersetzen();
        
        Collections.sort( werte, VorNachteilWert.NAME_COMPARATOR );
        
        aufbauenBenutzeroberflaeche();
        
        for ( int i = 0, n = werte.size() ; i < n ; ++i ) {
            
            update( i );
        }
    }
    
    /**
     * Macht einen Wert in {@link #werte} von einem Regelelement abhaengig.
     *
     * @param regelelement
     *            von dem der Wert abhaengig ist.
     * @param index
     *            des Wertes in {@link #werte}.
     */
    private void setAbhaengigkeit(
            final Object regelelement,
            final int index ) {

        if ( ! abhaengigkeiten.containsKey( regelelement ) ) {

            abhaengigkeiten.put( regelelement, new ArrayList< Integer>() );
        }

        abhaengigkeiten.get( regelelement ).add( index );
    }

    /**
     * Bestimmt den Held dessen Werte ueber die
     * Benutzerschnittstelle editiert werden.
     *
     * @param derHeld der aktuell erschaffenen wird.
     */
    final void setHeld( final Held derHeld ) {

        held = derHeld;

        if ( ! zeigtHeldenwerte ) {

            for ( int i = 0, n = werte.size() ; i < n ; ++i ) {

                final VorNachteilWert wert = werte.get( i );

                final VorNachteil vorNachteil = wert.getVorNachteil();

                if ( vorNachteil.hatStufen( wert ) ) {

                    final int min;

                    if ( held.hatVorgegeben( wert ) ) {

                        min = held.getWert( wert ).getStufe();

                    } else {

                        min = vorNachteil.getMindeststufe( held, wert );

                    }

                    // wert.setStufe( min );

                    indexSpinner.get( i ).setValue( min );
                }
            }
        }

        held.addObserver( this );

        anzahlGewaehlte = -1;
        // evtl. Heldenwerte anzeigen.
        update( null, null );

        for ( int i = werte.size()-1 ; i >= 0 ; --i ) {

            update( i );
        }
    }

    /* TODO *
     * @param panel
     *            auf den die Punkte fuer die Verbindungen verteilt werden.
     * /
    final void setPanelVerbindungen( final PanelVerbindungenListe panel ) {
        panelVerbindungen = panel;
    }
*/
    /**
     * Aendert die Werte die in der Liste dargestellt werden.
     *
     * @param dieWerte die angezeigt werden.
     */
    final void setWerte( final List< VorNachteilWert > dieWerte ) {

        werte.clear();

        werte.addAll( dieWerte );

        initAbhaengigkeiten();

        aufbauenBenutzeroberflaeche();
    }

    /**
     * Aktualisiert die Zeile eines Wertes.
     *
     * @param i
     *            Der Index des Wertes.
     */
    @SuppressWarnings("unchecked")
    private void update( final int i ) {

        final VorNachteilWert wert = werte.get( i );

        final VorNachteil vorNachteil = wert.getVorNachteil();

        final JLabel markierung = markierungen.get( i );

        if ( held.hatUngeeigneten( vorNachteil ) ) {

            markierung.setIcon( ICON_UNGEEIGNET );
            markierung.setToolTipText( I18n.i18n( UNGEEIGNET ) );

        } else if ( held.hatEmpfohlenen( vorNachteil ) ) {

            markierung.setIcon( ICON_EMPFOHLEN );
            markierung.setToolTipText( I18n.i18n( EMPFOHLEN ) );

        } else {

            markierung.setIcon( ICON_PLATZHALTER );
            markierung.setToolTipText( null );
        }

        if ( vorNachteil.hatStufen( wert )
          && held.hat( wert ) ) {

            final int stufe = held.getWert( wert ).getStufe();

            final JSpinner spinner = indexSpinner.get( i );

            spinner.setValue( stufe );

            // Das muss geschehen bevor der Wert ueberprueft wird.
            wert.setStufe( stufe );
        }

        final JCheckBox checkBox = vorNachteile.get( i );

        checkBox.setSelected( held.hat( wert ) );

        final KonfliktBeschreibung beschreibung = wert.pruefen( held );

        if ( beschreibung.enthaeltKonflikt() ) {

            checkBox.setForeground( Util.FEHLERFARBE );
            checkBox.setToolTipText( beschreibung.toHtml() );

        } else {

            checkBox.setForeground( getForeground() );
            checkBox.setToolTipText( null );

        }

        if ( held.hatVorgegeben( wert ) ) {

            checkBox.setFont( fetteSchrift );

        } else {

            checkBox.setFont( getFont() );

        }

        if ( wert.istVarianteAenderbar() ) {

            final JComboBox comboBox = varianten.get( i );

            final CharGenListModel< Variante, Variante > model;
            model = (CharGenListModel< Variante, Variante >) comboBox.getModel();

            final Variante variante = wert.getVariante();

            model.setElements( vorNachteil.getVarianten( held ), variante );

            checkBox.setEnabled( ! variante.isNull() );
        }

        final JLabel gp = gpKosten.get( i );

        gp.setText( String.format( 
                I18n.i18n( GP_KOSTEN ), 
                wert.getGpKosten( held ) ) );
    }

    /**
     * Aktualisiert die Anzeige wenn sich die Werte des Helden geaendert haben.
     *
     * @param o ungenutzt.
     * @param arg das Regelelement des Helden das geaendert wurde.
     */
    public void update( final Observable o, final Object arg ) {

        if ( zeigtHeldenwerte ) {

            final List< VorNachteilWert > gewaehlte;
            gewaehlte = held.getVorNachteile();

            if ( gewaehlte.size() != anzahlGewaehlte ) {

                // Die Liste der Werte die der Held besitzt hat sich gaendert.
                // Die ganze Liste muss neu aufgebaut werden.

                anzahlGewaehlte = gewaehlte.size();

                final List< VorNachteilWert > werte;
                werte = new ArrayList< VorNachteilWert >();

                werte.addAll( held.getFehlendeVorNachteile() );

                werte.addAll( gewaehlte );

                setWerte( werte );

                for ( int i = werte.size()-1 ; i >= 0 ; --i ) {

                    update( i );
                }
            }

        }

        if ( abhaengigkeiten.containsKey( arg ) ) {

            for ( final int index : abhaengigkeiten.get( arg ) ) {

                update( index );
            }
        }
    }

    /**
     * Wird aufgerufen wenn ein Vor-/Nachteil (ab)gewaehlt wird.
     *
     * @author Bernhard Jung
     */
    private class ActionDeAktivieren implements ActionListener {

        public final void actionPerformed( final ActionEvent e ) {

            final JCheckBox checkBox = (JCheckBox) e.getSource();

            final int index = Integer.valueOf( checkBox.getName() );

            final VorNachteilWert wert = werte.get( index );

            if ( checkBox.isSelected() ) {

                held.einfuegen( wert );

            } else {

                if ( held.hatVerbindungen( wert ) ) {

                    held.entfernenVerbindungen( wert );

                } else {

                    held.entfernen( wert );
                }
            }
        }
    }

    /**
     * Wird aufgerufen wenn die Variante eines Vor-/Nachteils geaendert wird.
     * Dies Klasse ist fuer Modus 1 gedacht. Siehe Dokumentation von
     * {@link PanelVorNachteileListe}.
     *
     * @author Bernhard Jung
     */
    private class ActionVariante implements ActionListener {

        public void actionPerformed( final ActionEvent e ) {

            final JComboBox comboBox = (JComboBox) e.getSource();

            Object variante = comboBox.getSelectedItem();

            if ( null == variante ) {
                // keine gueltige Variante gewaehlt. Abbruch.
                return;
            }

            if ( variante instanceof String ) {
                variante = ((String)variante).trim();
            }

            final int index = Integer.valueOf( comboBox.getName() );

            final VorNachteilWert wert = werte.get( index );
            
            wert.setVariante( variante );
            
            update( index );
/*
            if ( ! wert.getVariante().equals( variante ) ) {

                final VorNachteil vorNachteil = wert.getVorNachteil();

                if ( vorNachteil.hatFreieVarianten() 
                        && variante instanceof String ) {

                    vorNachteil.einfuegenVariante( variante.toString() );
                }

                wert.setVariante( variante );
            }

            update( index );
            */
        }
    }

    /* *
     * Wird aufgerufen wenn die Variante eines Vor-/Nachteils geaendert wird.
     * Diese Klasse ist fuer Modus 2 gedacht, in dem die Benutzeroberflaeche
     * die Vor-/Nachteile eines Helden anzeigt. Siehe Dokumentation von
     * {@link PanelVorNachteileListe}.
     *
     * @author Bernhard Jung
     * /
    private class ActionVarianteHeldenwerte implements ActionListener {

        public void actionPerformed( final ActionEvent e ) {

            final JComboBox comboBox = (JComboBox) e.getSource();

            Object variante = comboBox.getSelectedItem();

            if ( null == variante ) {
                // keine gueltige Variante gewaehlt. Abbruch.
                return;
            }

            if ( variante instanceof String ) {
                variante = ( (String)variante ).trim();
            }

            final int index = Integer.valueOf( comboBox.getName() );

            final VorNachteilWert wert = werte.get( index );

            // Der testwert ist da um mit  held.hat( testwert ) zu pruefen ob
            // der Held den Wert in dieser Variante schon besitzt.
            final VorNachteilWert testwert = wert.clone();

            testwert.setHeld( null );

            testwert.setVariante( variante );

            // nur wenn eine neue Variante gewaehlt wurde, ...
            if ( ! wert.getVariante().equals( variante )
                    // ... der Held die Variante noch nicht besitzt und ...
                    && ( ! held.hat( testwert ) )
                    // ... die Variante auch nicht vorgegeben ist.
                    // Warum dies letzte?
                    // Wenn dieser Ausdruck hier ausgewertet wird beutet dies
                    // der Held besitzt den Wert nicht, sonst haetten wir in der
                    // vorhergehenden Abfrage ein false erhalten und die &&
                    // Auswertung waere abgebrochen worden (&& ist lazy).
                    // Der Held besitzt den Wert also nicht. Ist der Wert
                    // vorgegeben so bedeutet dies, dass der Wert aktuell in der
                    // Liste der fehlenden, vorgegebenen Werte gespeichert ist.
                    // Werte die da gespeichert sind darf der Held nur erhalten,
                    // indem ein Wert mit held.einfuegen() hinzugefuegt wird und
                    // nicht indem man die Variante eines Wertes aendert den der
                    // Held schon besitzt.
                    // So wie es aktuell programmiert ist ist das halt so.
                    && ( ! held.hatVorgegeben( testwert ) ) ) {

                final VorNachteil vorNachteil = wert.getVorNachteil();

                if ( vorNachteil.hatFreieVarianten() ) {

                    vorNachteil.einfuegenVariante( variante.toString() );
                }

                if ( held.hatVerbindungen( wert ) ) {

                    panelVerbindungen.aendereVariante( wert, testwert );

                    held.setVerbindungenVariante( wert, variante );

                } else if ( held.hatVorgegeben( wert ) ) {
                    // Hier ist der Unterschied zur Klasse ActionVariante zu
                    // finden. Wenn die Variante eines fuer den Helden
                    // vorgegebenen Vor-/Nachteils geaendert wird, so bedeutet
                    // das in diesem Fall, dass die geaenderte Variante als
                    // vorgegeben gilt.
                    // Das ist fuer den Fall geadacht der bei der Profession
                    // Magier auftaucht. Der Magier erhaelt den Nachteil
                    // Verpflichtung (gegenueber dem Lehrmeister oder der
                    // Magiergilde), dabei muss vom Benutzer gewaehlt werden
                    // gegenueber wem genau die Verpflichtung besteht.

                    held.getVorgegeben( wert ).setVariante( variante );

                }

                wert.setVariante( variante );

            } else {

                // Wenn der Benutzer einen leeren String eingegeben und Return
                // gedrueckt hat, soll der urspruengliche Eintrag wieder im
                // Textfeld erscheinen um dem Benutzer zu zeigen, dass der leere
                // String nicht uebernommen wird.
                update( index );
            }
        }
    }
*/
    /**
     * Wird aufgerufen wenn der Wert eines Spinners geaendert wird.
     *
     * @author Bernhard Jung
     *
     */
    private final class StufeListener implements ChangeListener {
        /**
         * Wird aufgerufen wenn der Wert in einem der Spinner veraendert wird.
         */
        public void stateChanged( final ChangeEvent e ) {

            final JSpinner spinner = (JSpinner) e.getSource();

            final int i = Integer.valueOf( spinner.getName() );

            final VorNachteilWert wert = werte.get( i );

            final int stufe = (Integer) spinner.getValue();

            wert.setStufe( stufe );

            if ( held.hatVerbindungen( wert )
              && held.getVerbindungen( wert ).getStufe() > stufe ) {

                held.setVerbindungenStufe( wert );

            }

            if ( held.hat( wert )
              && ( held.getWert( wert ) != wert ) ) {

                held.getWert( wert ).setStufe( stufe );

            } else {

                update( i );

            }
        }
    }
}
