/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bernhardjung.dsaprogramm.gui.action;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.KeyStroke;

import de.bernhardjung.dsaprogramm.gui.Hauptpanel;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Util;

public abstract class CharGenAction
extends AbstractAction 
implements Uebersetzbar {
    
    /**
     * Das Hauptpanel der Anwendung.
     */
    protected final Hauptpanel hauptpanel;

    /** Der Name des Icons oder {@code null}, wenn es kein Icon gibt. */
    private final String l10nschluesselIcon;
    
    /**
     * Der Lokalisierungsschluessel, der verwendet wird um die Bezeichnung der
     * Action, die im Menue oder auf einem Knopf angezeigt wird, aus der
     * Lokalisierungsdatei zu laden.
     */
    private final String l10nschluesselName;
    
    /**
     * Der Lokalisierungsschluessel fuer die Taste mit der man einen 
     * Menueeintrag direkt anspringen kann, wenn man ein Menue aufgeklappt hat.
     */
    private final String l10nschluesselMnemonic;
    
    /**
     * Der Lokalisierungsschluessel fuer das Tastenkuerzel der Action, z.B. 
     * "Strg + S"
     */
    private final String l10nschluesselAccelerator;

    /**
     * @param derL10nschluesselName Der Schluessel der benutzt wird, um den
     *          Namen der Action, der im Menue angezeigt wird, aus der 
     *          Lokalisierungsdatei zu laden.
     */
    public CharGenAction(
            final Hauptpanel dasHauptpanel,
            final String derL10nschluesselName ) {
        
        this( dasHauptpanel, derL10nschluesselName, null, null, null );
    }

    /**
     * @param derL10nschluesselName Der Schluessel der benutzt wird, um den
     *          Namen der Action, der im Menue angezeigt wird, aus der 
     *          Lokalisierungsdatei zu laden.
     * @param icon Der Name des Icons.
     */
    public CharGenAction( 
            final Hauptpanel dasHauptpanel,
            final String derL10nschluesselName,
            final String icon ) {
        
        this( dasHauptpanel, derL10nschluesselName, icon, null, null );
    }

    /**
     * @param derL10nschluesselName Der Schluessel der benutzt wird, um den
     *          Namen der Action, der im Menue angezeigt wird, aus der 
     *          Lokalisierungsdatei zu laden.
     * @param derL10nschluesselAccelerator Der Schluessel der benutzt wird, um
     *          ein Tastenkuerzel wie "Strg + S" aus der Lokalisierungsdatei
     *          zu laden. Darf {@code null} sein.
     * @param derL10nschluesselMnemonic Der Schluessel fuer die Taste mit der
     *          man einen Menueeintrag direkt aktivieren kann, wenn man ein
     *          Menu aufgeklappt hat. Darf {@code null} sein.
     */
    public CharGenAction(
            final Hauptpanel dasHauptpanel,
            final String derL10nschluesselName,
            final String derL10nschluesselAccelerator,
            final String derL10nschluesselMnemonic ) {
        
        this ( dasHauptpanel,
                derL10nschluesselName,
                null,
                derL10nschluesselAccelerator,
                derL10nschluesselMnemonic );
    }

    /**
     * 
     * @param derL10nschluesselName
     * @param derL10nschluesselIcon Der Name des Icons.
     * @param derL10nschluesselAccelerator Der Schluessel der benutzt wird, um
     *          ein Tastenkuerzel wie "Strg + S" aus der Lokalisierungsdatei
     *          zu laden. Darf {@code null} sein.
     * @param derL10nschluesselMnemonic Der Schluessel fuer die Taste mit der
     *          man einen Menueeintrag direkt aktivieren kann, wenn man ein
     *          Menu aufgeklappt hat. Darf {@code null} sein.
     */
    public CharGenAction(
            final Hauptpanel dasHauptpanel, 
            final String derL10nschluesselName,
            final String derL10nschluesselIcon,
            final String derL10nschluesselAccelerator,
            final String derL10nschluesselMnemonic ) {
        
        hauptpanel = dasHauptpanel;
        
        l10nschluesselAccelerator = derL10nschluesselAccelerator;
        
        l10nschluesselIcon = derL10nschluesselIcon;
        
        l10nschluesselMnemonic = derL10nschluesselMnemonic;
        
        l10nschluesselName = derL10nschluesselName;

        putValue( NAME, I18n.i18n( l10nschluesselName ) );

        if ( null != l10nschluesselAccelerator ) {
        
            putValue( ACCELERATOR_KEY, KeyStroke.getKeyStroke( 
                    I18n.i18n( l10nschluesselAccelerator ) ) );
        }
        
        if ( null != l10nschluesselMnemonic ) {
            
            putValue( MNEMONIC_KEY,
                    Util.getKeyCode( I18n.i18n( l10nschluesselMnemonic ) ) );
        }
        
        if ( null != l10nschluesselIcon ) {
            
            putValue( SMALL_ICON, Util.ladeIcon( l10nschluesselIcon ) );
        }
    }

    /**
     * Wird aufgerufen, wenn die Action ausgeloest wurde.
     */
    protected abstract void actionPerformed();
    
    public void actionPerformed( ActionEvent e ) {
        
        try {
            
            hauptpanel.zeigeWartenMauszeiger();
        
            actionPerformed();
            
        } finally {
            
            hauptpanel.zeigeStandardMauszeiger();
        }
    }
    
    /**
     * Uebersetzt die Action neu.
     */
    public void neuUebersetzen() {

        putValue( NAME, I18n.i18n( l10nschluesselName ) );

        if ( null != l10nschluesselAccelerator ) {
        
            putValue( ACCELERATOR_KEY, KeyStroke.getKeyStroke( 
                    I18n.i18n( l10nschluesselAccelerator ) ) );
        }
        
        if ( null != l10nschluesselMnemonic ) {
            
            putValue( MNEMONIC_KEY,
                    Util.getKeyCode( I18n.i18n( l10nschluesselMnemonic ) ) );
        }
        
        if ( null != l10nschluesselIcon ) {
            
            putValue( SMALL_ICON, Util.ladeIcon( l10nschluesselIcon ) );
        }
    }
}
