/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.talente;

import java.util.ArrayList;
import java.util.List;


public class Kampftechnik extends Talent {

    /**
     * Dient zur Unterscheidung der Kampftechniken in Nahkampf- und
     * Fernkampf-Techniken.
     */
    static enum Art {
        /**
         * Fuer die Techniken des waffenlosen Kampfes und des bewaffneten
         * Nahkampfes.
         */
        NAHKAMPF,
        /** Fuer alle Fernkampf Techniken. */
        FERNKAMPF
    }

    /**
     * Gibt an ob es sich bei dem Talent um eine Nahkampf- oder
     * Fernkampffertigkeit handelt.
     */
    private final Art art;

    /**
     * Die Zahl die fuer die effektive Behinderung von der Behinderung abgezogen
     * wird.
     */
    private int eBe;

    /** Talente die alternativ, und mit Abzuegen, zum Fuehren einer Waffe
     * verwendet werden koennen. */
    private final List< Kampftechnik > verwandteFertigkeiten;

    /**
     * @param dieId der Kampftechnik.
     * @param derTyp {@code Typ.BASIS} oder {@code Typ.SPEZIAL}.
     * @param dieArt {@code Art.NAHKAMPF} oder {@code Art.FERNKAMPF}.
     */
    public Kampftechnik(
            final String dieId,
            final Typ derTyp,
            final Art dieArt ) {

        super( dieId, Talentgruppe.KAMPF, derTyp );

        art = dieArt;

        eBe = 0;

        verwandteFertigkeiten = new ArrayList< Kampftechnik >();
    }

    /**
     * @return die Zahl die fuer die effektive Behinderung von der Behinderung
     *         abgezogen wird.
     */
    public final int getEBe() {
        return eBe;
    }

    /**
     * @return die Talente, die alternativ und mit abzuegen zum Fuehren von
     *         Waffen verwendet werden koennen.
     */
    public final List< Kampftechnik > getVerwandteFertigkeiten() {
        return new ArrayList< Kampftechnik >( verwandteFertigkeiten );
    }
    /**
     * @return {@code true} wenn es sich bei dem Talent um eine Technik des
     *      waffenlosen Kampfes oder des bewaffneten Nahkampfes handelt, sonst
     *      {@code false}.
     */
    public final boolean istNahkampfTechnik() {
        return Art.NAHKAMPF == art;
    }

    /**
     * @return {@code true} wenn es sich bei dem Talent um eine
     *      Fernkampf Technik handelt, sonst {@code false},
     */
    public final boolean istFernkampfTechnik() {
        return Art.FERNKAMPF == art;
    }

    /**
     * @param eBe Die Zahl die fuer die effektive Behinderung von der
     *            Behinderung abgezogen wird.
     */
    final void setEBe( final int eBe ) {
        this.eBe = eBe;
    }

    /**
     * @param verwandte Fertigkeiten, die zum Fuehren einer Waffe aus dieser
     *             Kategorie verwendet werden koennen.
     */
    final void setVerwandteFertigkeiten(
            final Kampftechnik ... verwandte ) {

        verwandteFertigkeiten.clear();

        for ( final Kampftechnik technik : verwandte ) {
            verwandteFertigkeiten.add( technik );
        }
    }
}
