/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.talente;

import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.CH;
import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.FF;
import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.GE;
import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.IN;
import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.KK;
import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.KL;
import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.KO;
import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.MU;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

import de.bernhardjung.dsaprogramm.basis.Eigenschaftsprobe;
import de.bernhardjung.dsaprogramm.basis.ElementMitNameImpl;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.basis.SKT;
import de.bernhardjung.dsaprogramm.basis.Steigerbar;
import de.bernhardjung.dsaprogramm.basis.varianten.Variante;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.talente.Kampftechnik.Art;
import de.bernhardjung.dsaprogramm.talente.kosten.Kosten;
import de.bernhardjung.dsaprogramm.talente.stufen.Stufen;
import de.bernhardjung.dsaprogramm.talente.voraussetzungen.Voraussetzung;
import de.bernhardjung.dsaprogramm.util.Util;

public class Talent
extends ElementMitNameImpl
implements Steigerbar {
    
    /**
     * Sortiert die Talente 1. nach Talentgruppe, 2. alphabetisch nach Namen.
     */
    public static final Comparator< Talent > TALENTGRUPPEN_COMPARATOR =
        new Comparator< Talent >() {
            public int compare( Talent t1, Talent t2 ) {

                int ergebnis = t1.getTalentgruppe().compareTo( t2.getTalentgruppe() );

                if ( ergebnis != 0 ) {

                    return ergebnis;
                }

                return Util.compare( t1.getName(), t2.getName() );
            }
    };

    static enum Typ {
        /** Der Typ fuer die Basis-Talente. */
        BASIS,
        /** Der Typ fuer die Spezial-Talente. */
        SPEZIAL,
        /** Der Typ fuer die Gaben. */
        GABE,
        /** Ein extra Typ fuer die Ritualkenntnis. */
        RITUALKENNTNIS
    };

    /** Eine Liste aller Basis-Talente. */
    private static final Map< String, Talent > BASIS_TALENTE =
        new HashMap< String, Talent >();

    /** Eine Lister aller Fernkampf-Techniken. */
    private static final Map< String, Kampftechnik > FERNKAMPF_TALENTE =
        new HashMap< String, Kampftechnik >();

    /** Eine Liste aller normalen Talente. */
    private static final Map< String, Talent > TALENTE_OHNE_GABEN =
        new HashMap< String, Talent >();

    /** Eine Map mit allten Talenten, auch Gaben. */
    private static final Map< String, Talent > TALENTE_MAP =
        new HashMap< String, Talent >();

    /*
     * KAMPFTECHNIK-TALENTE
     */
    public static final Kampftechnik ANDERTHALBHAENDER =
        new Kampftechnik( "Talent.Anderthalbhaender",
                Typ.SPEZIAL,
                Art.NAHKAMPF );

    public static final Kampftechnik ARMBRUST =
        new Kampftechnik( "Talent.Armbrust",
                Typ.SPEZIAL,
                Art.FERNKAMPF );

    public static final Kampftechnik BOGEN =
        new Kampftechnik( "Talent.Bogen",
                Typ.SPEZIAL,
                Art.FERNKAMPF );

    public static final Kampftechnik DOLCHE =
        new Kampftechnik( "Talent.Dolche",
                Typ.BASIS,
                Art.NAHKAMPF );

    public static final Kampftechnik FECHTWAFFEN =
        new Kampftechnik( "Talent.Fechtwaffen",
                Typ.SPEZIAL,
                Art.NAHKAMPF );

    public static final Kampftechnik HIEBWAFFEN =
        new Kampftechnik( "Talent.Hiebwaffen",
                Typ.BASIS,
                Art.NAHKAMPF );

    public static final Kampftechnik INFANTERIEWAFFEN =
        new Kampftechnik( "Talent.Infanteriewaffen",
                Typ.SPEZIAL,
                Art.NAHKAMPF );

    public static final Kampftechnik KETTENWAFFEN =
        new Kampftechnik( "Talent.Kettenwaffen",
                Typ.SPEZIAL,
                Art.NAHKAMPF );

    public static final Kampftechnik LANZENREITEN =
        new Kampftechnik( "Talent.Lanzenreiten",
                Typ.SPEZIAL,
                Art.NAHKAMPF );

    public static final Kampftechnik RAUFEN =
        new Kampftechnik( "Talent.Raufen",
                Typ.BASIS,
                Art.NAHKAMPF );

    public static final Kampftechnik RINGEN =
        new Kampftechnik( "Talent.Ringen",
                Typ.BASIS,
                Art.NAHKAMPF );

    public static final Kampftechnik SAEBEL =
        new Kampftechnik( "Talent.Saebel",
                Typ.BASIS,
                Art.NAHKAMPF );

    public static final Kampftechnik SCHWERTER =
        new Kampftechnik( "Talent.Schwerter",
                Typ.SPEZIAL,
                Art.NAHKAMPF );

    public static final Kampftechnik SPEERE =
        new Kampftechnik( "Talent.Speere",
                Typ.SPEZIAL,
                Art.NAHKAMPF );

    public static final Kampftechnik STAEBE =
        new Kampftechnik( "Talent.Staebe",
                Typ.SPEZIAL,
                Art.NAHKAMPF );

    public static final Kampftechnik WURFBEILE =
        new Kampftechnik( "Talent.Wurfbeile",
                Typ.SPEZIAL,
                Art.FERNKAMPF );

    public static final Kampftechnik WURFMESSER =
        new Kampftechnik( "Talent.Wurfmesser",
                Typ.BASIS,
                Art.FERNKAMPF );

    public static final Kampftechnik WURFSPEERE =
        new Kampftechnik( "Talent.Wurfspeere",
                Typ.SPEZIAL,
                Art.FERNKAMPF );

    public static final Kampftechnik ZWEIHAND_HIEBWAFFEN =
        new Kampftechnik( "Talent.ZweihandHiebwaffen",
                Typ.SPEZIAL,
                Art.NAHKAMPF );

    public static final Kampftechnik ZWEIHANDSCHWERTER =
        new Kampftechnik( "Talent.Zweihandschwerter",
                Typ.SPEZIAL,
                Art.NAHKAMPF );

    /*
     * KOERPERLICHE TALENTE
     */
    public static final Talent AKROBATIK = new Talent(
            "Talent.Akrobatik",
            Talentgruppe.KOERPER,
            Typ.SPEZIAL );

    public static final Talent ATHLETIK = new Talent(
            "Talent.Athletik",
            Talentgruppe.KOERPER,
            Typ.BASIS );

    public static final Talent GAUKELEIEN = new Talent(
            "Talent.Gaukeleien",
            Talentgruppe.KOERPER,
            Typ.SPEZIAL );

    public static final Talent KLETTERN = new Talent(
            "Talent.Klettern",
            Talentgruppe.KOERPER,
            Typ.BASIS );

    public static final Talent KOERPERBEHERRSCHUNG = new Talent(
            "Talent.Koerperbeherrschung",
            Talentgruppe.KOERPER,
            Typ.BASIS );

    public static final Talent REITEN = new Talent(
            "Talent.Reiten",
            Talentgruppe.KOERPER,
            Typ.SPEZIAL );

    public static final Talent SCHLEICHEN = new Talent(
            "Talent.Schleichen",
            Talentgruppe.KOERPER,
            Typ.BASIS);

    public static final Talent SCHWIMMEN = new Talent(
            "Talent.Schwimmen",
            Talentgruppe.KOERPER,
            Typ.BASIS );

    public static final Talent SELBSTBEHERRSCHUNG = new Talent(
            "Talent.Selbstbeherrschung",
            Talentgruppe.KOERPER,
            Typ.BASIS );

    public static final Talent SICH_VERSTECKEN = new Talent(
            "Talent.SichVerstecken",
            Talentgruppe.KOERPER,
            Typ.BASIS );

    public static final Talent SINGEN = new Talent(
            "Talent.Singen",
            Talentgruppe.KOERPER,
            Typ.BASIS );

    public static final Talent SINNENSCHAERFE = new Talent(
            "Talent.Sinnenschaerfe",
            Talentgruppe.KOERPER,
            Typ.BASIS );

    public static final Talent STIMMEN_IMITIEREN = new Talent(
            "Talent.StimmenImitieren",
            Talentgruppe.KOERPER,
            Typ.SPEZIAL );

    public static final Talent TANZEN = new Talent(
            "Talent.Tanzen",
            Talentgruppe.KOERPER,
            Typ.BASIS );

    public static final Talent TASCHENDIEBSTAHL = new Talent(
            "Talent.Taschendiebstahl",
            Talentgruppe.KOERPER,
            Typ.SPEZIAL );

    public static final Talent ZECHEN = new Talent(
            "Talent.Zechen",
            Talentgruppe.KOERPER,
            Typ.BASIS );

    /*
     * GESELLSCHAFTLICHE TALENTE
     */
    public static final Talent BETOEREN = new Talent(
            "Talent.Betoeren",
            Talentgruppe.GESELLSCHAFT,
            Typ.SPEZIAL );

    public static final Talent ETIKETTE = new Talent(
            "Talent.Etikette",
            Talentgruppe.GESELLSCHAFT,
            Typ.SPEZIAL );

    public static final Talent GASSENWISSEN = new Talent(
            "Talent.Gassenwissen",
            Talentgruppe.GESELLSCHAFT,
            Typ.SPEZIAL );

    public static final Talent LEHREN = new Talent(
            "Talent.Lehren",
            Talentgruppe.GESELLSCHAFT,
            Typ.SPEZIAL );

    public static final Talent MENSCHENKENNTNIS = new Talent(
            "Talent.Menschenkenntnis",
            Talentgruppe.GESELLSCHAFT,
            Typ.BASIS );

    public static final Talent SICH_VERKLEIDEN = new Talent(
            "Talent.SichVerkleiden",
            Talentgruppe.GESELLSCHAFT,
            Typ.SPEZIAL );

    public static final Talent UEBERREDEN = new Talent(
            "Talent.Ueberreden",
            Talentgruppe.GESELLSCHAFT,
            Typ.BASIS );

    public static final Talent UEBERZEUGEN = new Talent(
            "Talent.Ueberzeugen",
            Talentgruppe.GESELLSCHAFT,
            Typ.SPEZIAL );

    /*
     * NATUR-TALENTE
     */
    public static final Talent FAEHRTENSUCHEN = new Talent(
            "Talent.Faehrtensuchen",
            Talentgruppe.NATUR,
            Typ.BASIS );

    public static final Talent FALLENSTELLEN = new Talent(
            "Talent.Fallenstellen",
            Talentgruppe.NATUR,
            Typ.SPEZIAL );

    public static final Talent FESSELN_ENTFESSELN = new Talent(
            "Talent.FesselnEntfesseln",
            Talentgruppe.NATUR,
            Typ.SPEZIAL );

    public static final Talent FISCHEN_ANGELN = new Talent(
            "Talent.FischenAngeln",
            Talentgruppe.NATUR,
            Typ.SPEZIAL );

    public static final Talent ORIENTIERUNG = new Talent(
            "Talent.Orientierung",
            Talentgruppe.NATUR,
            Typ.BASIS );

    public static final Talent WETTERVORHERSAGE = new Talent(
            "Talent.Wettervorhersage",
            Talentgruppe.NATUR,
            Typ.SPEZIAL );

    public static final Talent WILDNISLEBEN = new Talent(
            "Talent.Wildnisleben",
            Talentgruppe.NATUR,
            Typ.BASIS );

    /*
     * WISSENS-TALENTE
     */
    public static final Talent ANATOMIE = new Talent(
            "Talent.Anatomie",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent BRETT_WUERFELSPIELE = new Talent(
            "Talent.BrettWuerfelspiele",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent GEOGRAPHIE = new Talent(
            "Talent.Geographie",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent GESCHICHTSWISSEN = new Talent(
            "Talent.Geschichtswissen",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent GESTEINSKUNDE = new Talent(
            "Talent.Gesteinskunde",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent GOETTER_KULTE = new Talent(
            "Talent.GoetterKulte",
            Talentgruppe.WISSEN,
            Typ.BASIS );

    public static final Talent HERALDIK = new Talent(
            "Talent.Heraldik",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent KRIEGSKUNST = new Talent(
            "Talent.Kriegskunst",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent MAGIEKUNDE = new Talent(
            "Talent.Magiekunde",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent MECHANIK = new Talent(
            "Talent.Mechanik",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent PFLANZENKUNDE = new Talent(
            "Talent.Pflanzenkunde",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent RECHNEN = new Talent(
            "Talent.Rechnen",
            Talentgruppe.WISSEN,
            Typ.BASIS );

    public static final Talent RECHTSKUNDE = new Talent(
            "Talent.Rechtskunde",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent SAGEN_LEGENDEN = new Talent(
            "Talent.SagenLegenden",
            Talentgruppe.WISSEN,
            Typ.BASIS );

    public static final Talent SCHAETZEN = new Talent(
            "Talent.Schaetzen",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent SPRACHENKUNDE = new Talent(
            "Talent.Sprachenkunde",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent STAATSKUNST = new Talent(
            "Talent.Staatskunst",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent STERNKUNDE = new Talent(
            "Talent.Sternkunde",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent TIERKUNDE = new Talent(
            "Talent.Tierkunde",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    /*
     * SPRACHEN
     */
    public static final Sprache ALAANI =
        new Sprache( "Talent.Alaani" );

    public static final Sprache ASDHARIA =
        new Sprache( "Talent.Asdharia" );

    public static final Sprache ATAK =
        new Sprache( "Talent.Atak" );

    public static final Sprache BOSPARANO =
        new Sprache( "Talent.Bosparano" );

    public static final Sprache FUECHSISCH =
        new Sprache( "Talent.Fuechsisch" );

    public static final Sprache GARETHI =
        new Sprache( "Talent.Garethi" );

    public static final Sprache GOBLINISCH =
        new Sprache( "Talent.Goblinisch" );

    public static final Sprache ISDIRA =
        new Sprache( "Talent.Isdira" );

    public static final Sprache MOHISCH =
        new Sprache( "Talent.Mohisch" );

    public static final Sprache NUJUKA =
        new Sprache( "Talent.Nujuka" );

    public static final Sprache OLOARKH =
        new Sprache( "Talent.Oloarkh" );

    public static final Sprache OLOGHAIJAN =
        new Sprache( "Talent.Ologhaijan" );

    public static final Sprache ROGOLAN =
        new Sprache( "Talent.Rogolan" );

    public static final Sprache RSSAHH =
        new Sprache( "Talent.Rssahh" );

    public static final Sprache THORWALSCH =
        new Sprache( "Talent.Thorwalsch" );

    public static final Sprache TULAMIDYA =
        new Sprache( "Talent.Tulamidya" );

    public static final Sprache UR_TULAMIDYA =
        new Sprache( "Talent.UrTulamidya" );

    public static final Sprache ZHAYAD =
        new Sprache( "Talent.Zhayad" );

    /*
     *  SCHRIFTEN
     */
    public static final Schrift ASDHARIA_SCHRIFT =
        new Schrift( "Talent.AsdhariaSchrift" );

    public static final Schrift CHRMK =
        new Schrift( "Talent.Chrmk" );

    public static final Schrift GLYPHEN_VON_UNAU =
        new Schrift( "Talent.GlyphenVonUnau" );

    public static final Schrift HJALDINGSCHE_RUNEN =
        new Schrift( "Talent.HjaldingscheRunen" );

    public static final Schrift ISDIRA_SCHRIFT =
        new Schrift( "Talent.IsdiraSchrift" );

    public static final Schrift KUSLIKER_ZEICHEN =
        new Schrift( "Talent.KuslikerZeichen" );

    public static final Schrift NANDURIA =
        new Schrift( "Talent.Nanduria" );

    public static final Schrift ROGOLAN_SCHRIFT =
        new Schrift( "Talent.RogolanSchrift" );

    public static final Schrift TULAMIDYA_SCHRIFT =
        new Schrift( "Talent.TulamidyaSchrift" );

    public static final Schrift UR_TULAMIDYA_SCHRIFT =
        new Schrift( "Talent.UrTulamidyaSchrift" );

    public static final Schrift ZHAYAD_SCHRIFT =
        new Schrift( "Talent.ZhayadSchrift" );

    /*
     * HANDWERKS-TALENTE
     */
    public static final Talent ABRICHTEN = new Talent(
            "Talent.Abrichten",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent ACKERBAU = new Talent(
            "Talent.Ackerbau",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent ALCHIMIE = new Talent(
            "Talent.Alchimie",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent BERGBAU = new Talent(
            "Talent.Bergbau",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent BOGENBAU = new Talent(
            "Talent.Bogenbau",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent BOOTE_FAHREN = new Talent(
            "Talent.BooteFahren",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent FAHRZEUG_LENKEN = new Talent(
            "Talent.FahrzeugLenken",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent FALSCHSPIEL = new Talent(
            "Talent.Falschspiel",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent FEINMECHANIK = new Talent(
            "Talent.Feinmechanik",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent FLEISCHER = new Talent(
            "Talent.Fleischer",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent GERBER_KUERSCHNER = new Talent(
            "Talent.GerberKuerschner",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent GROBSCHMIED = new Talent(
            "Talent.Grobschmied",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent HEILKUNDE_GIFT = new Talent(
            "Talent.HeilkundeGift",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent HEILKUNDE_KRANKHEITEN = new Talent(
            "Talent.HeilkundeKrankheiten",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent HEILKUNDE_SEELE = new Talent(
            "Talent.HeilkundeSeele",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent HEILKUNDE_WUNDEN = new Talent(
            "Talent.HeilkundeWunden",
            Talentgruppe.HANDWERK,
            Typ.BASIS );

    public static final Talent HOLZBEARBEITUNG = new Talent(
            "Talent.Holzbearbeitung",
            Talentgruppe.HANDWERK,
            Typ.BASIS );

    public static final Talent KARTOGRAPHIE = new Talent(
            "Talent.Kartographie",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent KOCHEN = new Talent(
            "Talent.Kochen",
            Talentgruppe.HANDWERK,
            Typ.BASIS );

    public static final Talent LEDERARBEITEN = new Talent(
            "Talent.Lederarbeiten",
            Talentgruppe.HANDWERK,
            Typ.BASIS );

    public static final Talent MALEN_ZEICHNEN = new Talent(
            "Talent.MalenZeichnen",
            Talentgruppe.HANDWERK,
            Typ.BASIS );

    public static final Talent MUSIZIEREN = new Talent(
            "Talent.Musizieren",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent SCHLOESSER_KNACKEN = new Talent(
            "Talent.SchloesserKnacken",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent SCHNEIDERN = new Talent(
            "Talent.Schneidern",
            Talentgruppe.HANDWERK,
            Typ.BASIS );

    public static final Talent SEEFAHRT = new Talent(
            "Talent.Seefahrt",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent STEINMETZ = new Talent(
            "Talent.Steinmetz",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent STEINSCHNEIDER = new Talent(
            "Talent.Steinschneider",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent TAETOWIEREN = new Talent(
            "Talent.Taetowieren",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent ZIMMERMANN = new Talent(
            "Talent.Zimmermann",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    /*
     * GABEN
     */
    public static final Talent GEFAHRENINSTINKT = new Talent(
            "Talent.Gefahreninstinkt",
            Talentgruppe.GABEN,
            Typ.GABE );

    public static final Talent MAGIEGESPUER = new Talent(
            "Talent.Magiegespuer",
            Talentgruppe.GABEN,
            Typ.GABE );

    public static final Talent PROPHEZEIEN = new Talent(
            "Talent.Prophezeien",
            Talentgruppe.GABEN,
            Typ.GABE );

    public static final Talent ZWERGENNASE = new Talent(
            "Talent.Zwergennase",
            Talentgruppe.GABEN,
            Typ.GABE );

    /*
     * RITUALKENNTNIS
     */
    public static final Talent RITUALKENNTNIS = new Talent(
            "Talent.Ritualkenntnis",
            Talentgruppe.RITUALKENNTNIS,
            Typ.RITUALKENNTNIS );

    /**
     * @param id des gesuchten Talents.
     * @return das Talent mit der uebergebenen Id.
     * @throws IllegalArgumentException wenn kein Talent mit der uebergebenen
     *      Id existiert.
     */
    public static Talent getTalent( final String id ) {

        if ( TALENTE_MAP.containsKey( id ) ) {

            return TALENTE_MAP.get( id );

        } else {

            throw new IllegalArgumentException(
                    "Es existiert kein Talent mit der Id " + id );

        }
    }
    /**
     * Initialisiert alle Talente.
     * Dies Methode muss einmal aufgerufen werden bevor man die Talente im
     * Programm verwenden kann.
     */
    public static void initialisieren() {

        /*
         * KAMPFTECHNIK-TALENTE
         */
        initialisiereKampftechnik( ANDERTHALBHAENDER, SKT.E, 2 );
        ANDERTHALBHAENDER.setSpezialisierungen(
                "Waffe.Anderthalbhaender",
                "Waffe.Rondrakamm",
                "Waffe.Tuzakmesser" );
        ANDERTHALBHAENDER.setVerwandteFertigkeiten(
                SCHWERTER, ZWEIHANDSCHWERTER );

        initialisiereKampftechnik( ARMBRUST, SKT.C, 5 );
        ARMBRUST.setSpezialisierungen(
                "Waffe.Ballaester",
                "Waffe.Eisenwalder",
                "Waffe.LeichteArmbrust",
                "Waffe.Windenarmbrust" );
        ARMBRUST.setVerwandteFertigkeiten( BOGEN );

        initialisiereKampftechnik( BOGEN, SKT.E, 3 );
        BOGEN.einfuegen( Voraussetzung.NICHT_EINARMIG );
        BOGEN.setSpezialisierungen(
                "Waffe.Elfenbogen",
                "Waffe.Kompositbogen",
                "Waffe.Kriegsbogen",
                "Waffe.Kurzbogen",
                "Waffe.Langbogen" );

        initialisiereKampftechnik( DOLCHE, SKT.D, 1 );
        DOLCHE.setSpezialisierungen(
                "Waffe.Dolch",
                "Waffe.Drachenzahn",
                "Waffe.Jagdmesser",
                "Waffe.Langdolch",
                "Waffe.Messer",
                "Waffe.SchwererDolch",
                "Waffe.Wurfdolch",
                "Waffe.Wurfmesser" );
        DOLCHE.setVerwandteFertigkeiten( FECHTWAFFEN );

        initialisiereKampftechnik( FECHTWAFFEN, SKT.E, 1 );
        FECHTWAFFEN.setSpezialisierungen(
                "Waffe.Degen",
                "Waffe.Florett",
                "Waffe.Rapier" );
        FECHTWAFFEN.setVerwandteFertigkeiten( DOLCHE, SCHWERTER );

        initialisiereKampftechnik( HIEBWAFFEN, SKT.D, 4 );
        HIEBWAFFEN.setSpezialisierungen(
                "Waffe.Beil",
                "Waffe.Brabakbengel",
                "Waffe.Fackel",
                "Waffe.Keule",
                "Waffe.Knueppel",
                "Waffe.Lindwurmschlaeger",
                "Waffe.Rabenschnabel",
                "Waffe.Sichel",
                "Waffe.Skraja",
                "Waffe.Streitaxt",
                "Waffe.Streitkolben",
                "Waffe.Stuhlbein",
                "Waffe.Wurfbeil" );
        HIEBWAFFEN.setVerwandteFertigkeiten( SAEBEL );

        initialisiereKampftechnik( INFANTERIEWAFFEN, SKT.D, 3 );
        INFANTERIEWAFFEN.setSpezialisierungen(
                "Waffe.Glefe",
                "Waffe.Hakenspiess",
                "Waffe.Hellebarde",
                "Waffe.Partisane",
                "Waffe.Sense",
                "Waffe.Sturmsense",
                "Waffe.Wurmspiess" );
        INFANTERIEWAFFEN.setVerwandteFertigkeiten( SPEERE, ZWEIHAND_HIEBWAFFEN);

        initialisiereKampftechnik( KETTENWAFFEN, SKT.D, 3 );
        KETTENWAFFEN.setSpezialisierungen(
                "Waffe.Morgenstern",
                "Waffe.Ochsenherde",
                "Waffe.Ogerschelle" );

        initialisiereKampftechnik( LANZENREITEN, SKT.E, 0 );

        initialisiereKampftechnik( RAUFEN, SKT.C, 0 );

        initialisiereKampftechnik( RINGEN, SKT.D, 0 );

        initialisiereKampftechnik( SAEBEL, SKT.D, 2 );
        SAEBEL.setSpezialisierungen(
                "Waffe.Entermesser",
                "Waffe.Haumesser",
                "Waffe.Khunchomer",
                "Waffe.Saebel",
                "Waffe.Waqqif" );
        SAEBEL.setVerwandteFertigkeiten( SCHWERTER, HIEBWAFFEN );

        initialisiereKampftechnik( SCHWERTER, SKT.E, 2 );
        SCHWERTER.setSpezialisierungen(
                "Waffe.Kurzschwert",
                "Waffe.KuslikerSaebel",
                "Waffe.Schwert" );
        SCHWERTER.setVerwandteFertigkeiten( SAEBEL, FECHTWAFFEN,
                ANDERTHALBHAENDER );

        initialisiereKampftechnik( SPEERE, SKT.D, 3 );
        SPEERE.setSpezialisierungen(
                "Waffe.Dreizack",
                "Waffe.Dschadra",
                "Waffe.Holzspeer",
                "Waffe.Jagdspiess",
                "Waffe.Speer",
                "Waffe.Stossspeer",
                "Waffe.Wurfspeer" );
        SPEERE.setVerwandteFertigkeiten( INFANTERIEWAFFEN );

        initialisiereKampftechnik( STAEBE, SKT.D, 2 );
        STAEBE.setSpezialisierungen(
                "Waffe.Kampfstab",
                "Waffe.Magierstab",
                "Waffe.Zweililien" );
        STAEBE.setVerwandteFertigkeiten( SPEERE );

        initialisiereKampftechnik( WURFBEILE, SKT.C, 2 );
        WURFBEILE.setSpezialisierungen(
                "Waffe.Schneidzahn",
                "Waffe.Wurfbeil" );
        WURFBEILE.setVerwandteFertigkeiten( WURFSPEERE, WURFMESSER );

        initialisiereKampftechnik( WURFMESSER, SKT.C, 3 );
        WURFMESSER.setSpezialisierungen(
                "Waffe.Borndorn",
                "Waffe.Dolch",
                "Waffe.Wurfdolch",
                "Waffe.Wurfmesser",
                "Waffe.WurfscheibeRing" );
        WURFMESSER.setVerwandteFertigkeiten( WURFBEILE, WURFSPEERE );

        initialisiereKampftechnik( WURFSPEERE, SKT.C, 2 );
        WURFSPEERE.setSpezialisierungen(
                "Waffe.Holzspeer",
                "Waffe.Speer",
                "Waffe.Wurfspeer",
                "Waffe.Dschadra" );
        WURFSPEERE.setVerwandteFertigkeiten( WURFBEILE );

        initialisiereKampftechnik( ZWEIHAND_HIEBWAFFEN, SKT.D, 3 );
        ZWEIHAND_HIEBWAFFEN.einfuegen( Voraussetzung.NICHT_EINARMIG );
        ZWEIHAND_HIEBWAFFEN.setSpezialisierungen(
                "Waffe.Barbarenstreitaxt",
                "Waffe.Felsspalter",
                "Waffe.Holzfaelleraxt",
                "Waffe.Kriegshammer",
                "Waffe.Orknase",
                "Waffe.Spitzhacke",
                "Waffe.Vorschlaghammer",
                "Waffe.WarunkerHammer",
                "Waffe.Zwergenschlaegel" );
        ZWEIHAND_HIEBWAFFEN.setVerwandteFertigkeiten( HIEBWAFFEN,
                INFANTERIEWAFFEN );

        initialisiereKampftechnik( ZWEIHANDSCHWERTER, SKT.E, 2 );
        ZWEIHANDSCHWERTER.einfuegen( Voraussetzung.NICHT_EINARMIG );
        ZWEIHANDSCHWERTER.setSpezialisierungen(
                "Waffe.Andergaster",
                "Waffe.Boronssichel",
                "Waffe.Doppelkhunchomer",
                "Waffe.Rondrakamm",
                "Waffe.Zweihaender" );
        ZWEIHANDSCHWERTER.setVerwandteFertigkeiten( ANDERTHALBHAENDER,
                SCHWERTER, SAEBEL );

        /*
         * KOERPERLICHE TALENTE
         */
        initialisieren( AKROBATIK,
                        Kosten.SCHLANGENMENSCH,
                        SKT.D,
                        Stufen.TALENT,
                        MU, GE, KK );
        AKROBATIK.einfuegen( Voraussetzung.KOERPERBEHERSCHUNG_4 );
        AKROBATIK.setSpezialisierungen(
                "Talent.Spezialisierung.Balancieren",
                "Talent.Spezialisierung.Bodenakrobatik",
                "Talent.Spezialisierung.Schwingen",
                "Talent.Spezialisierung.Spruenge",
                "Talent.Spezialisierung.Winden" );

        initialisieren( ATHLETIK, SKT.D, GE, KO, KK );
        ATHLETIK.setSpezialisierungen(
                "Talent.Spezialisierung.Hochsprung",
                "Talent.Spezialisierung.Kraftakte",
                "Talent.Spezialisierung.Langlauf",
                "Talent.Spezialisierung.Sprint",
                "Talent.Spezialisierung.Weitsprung" );

        initialisieren( GAUKELEIEN,
                        Kosten.SCHLANGENMENSCH,
                        SKT.D,
                        Stufen.TALENT,
                        MU, CH, FF );
        GAUKELEIEN.setSpezialisierungen(
                "Talent.Spezialisierung.Bauchreden",
                "Talent.Spezialisierung.Feuerkunst",
                "Talent.Spezialisierung.Jonglieren",
                "Talent.Spezialisierung.Possenreissen",
                "Talent.Spezialisierung.Taschenspielereien" );

        initialisieren( KLETTERN, SKT.D, MU, GE, KK );
        KLETTERN.setSpezialisierungen(
                "Talent.Spezialisierung.Bergsteigen",
                "Talent.Spezialisierung.Eisklettern",
                "Talent.Spezialisierung.Freiklettern",
                "Talent.Spezialisierung.Seilklettern" );

        initialisieren( KOERPERBEHERRSCHUNG,
                        Kosten.SCHLANGENMENSCH,
                        SKT.D,
                        Stufen.TALENT,
                        MU, IN, GE );
        KOERPERBEHERRSCHUNG.setSpezialisierungen(
                "Talent.Spezialisierung.Fallen",
                "Talent.Spezialisierung.Spruenge",
                "Talent.Spezialisierung.Standfestigkeit" );

        initialisieren( REITEN, SKT.D, CH, GE, KK );
        REITEN.setSpezialisierungen(
                "Talent.Spezialisierung.PferdEselMuli",
                "Talent.Spezialisierung.Kamel",
                "Talent.Spezialisierung.Strauss",
                "Talent.Spezialisierung.Reitechse",
                "Talent.Spezialisierung.Flugechse",
                "Talent.Spezialisierung.Hippogriff",
                "Talent.Spezialisierung.GoblinischesReitWildschwein" );

        initialisieren( SCHLEICHEN,
                        Kosten.SCHLANGENMENSCH,
                        SKT.D,
                        Stufen.TALENT,
                        MU, IN, GE );
        SCHLEICHEN.setSpezialisierungen(
                "Talent.Spezialisierung.EisSchnee",
                "Talent.Spezialisierung.Gebaeude",
                "Talent.Spezialisierung.Geroell",
                "Talent.Spezialisierung.Strassen",
                "Talent.Spezialisierung.WaldDschungel" );

        initialisieren( SCHWIMMEN, SKT.D, GE, KO, KK );
        SCHWIMMEN.setSpezialisierungen(
                "Talent.Spezialisierung.Langstreckenschwimmen",
                "Talent.Spezialisierung.Schnellschwimmen",
                "Talent.Spezialisierung.Tauchen",
                "Talent.Spezialisierung.Unterwasserkampf" );

        initialisieren( SELBSTBEHERRSCHUNG, SKT.D, MU, KO, KK );
        SELBSTBEHERRSCHUNG.setSpezialisierungen(
                "Talent.Spezialisierung.ErschoepfungIgnorieren",
                "Talent.Spezialisierung.SchmerzUnterdruecken",
                "Talent.Spezialisierung.StoerungenIgnorieren",
                "Talent.Spezialisierung.WundenIgnorieren" );

        initialisieren( SICH_VERSTECKEN,
                        Kosten.SCHLANGENMENSCH,
                        SKT.D,
                        Stufen.TALENT,
                        MU, IN, GE );
        SICH_VERSTECKEN.setSpezialisierungen(
                "Talent.Spezialisierung.Gebaeude",
                "Talent.Spezialisierung.Gebirge",
                "Talent.Spezialisierung.Grasland",
                "Talent.Spezialisierung.EisSchnee",
                "Talent.Spezialisierung.Stadt",
                "Talent.Spezialisierung.WaldDschungel",
                "Talent.Spezialisierung.Wueste" );

        initialisieren( SINGEN, SKT.D, IN, CH, KO );
        SINGEN.setSpezialisierungen(
                "Talent.Spezialisierung.Balladenvortrag",
                "Talent.Spezialisierung.Chorgesang",
                "Talent.Spezialisierung.Mehrstimmigkeit",
                "Talent.Spezialisierung.Operngesang" );

        initialisieren( SINNENSCHAERFE, SKT.D, KL, IN, IN, FF );
        SINNENSCHAERFE.setSpezialisierungen(
                "Talent.Spezialisierung.Hoeren",
                "Talent.Spezialisierung.RiechenSchmecken",
                "Talent.Spezialisierung.Sehen",
                "Talent.Spezialisierung.Tasten" );

        initialisieren( STIMMEN_IMITIEREN, SKT.D,  KL, IN, CH );
        STIMMEN_IMITIEREN.einfuegen( Voraussetzung.SINNENSCHAERFE_4 );
        STIMMEN_IMITIEREN.setSpezialisierungen(
                "Talent.Spezialisierung.Haustiere",
                "Talent.Spezialisierung.Jagdwild",
                "Talent.Spezialisierung.MenschlicheStimmen",
                "Talent.Spezialisierung.Raubtiere",
                "Talent.Spezialisierung.Voegel" );

        initialisieren( TANZEN,
                        Kosten.SCHLANGENMENSCH,
                        SKT.D,
                        Stufen.TALENT,
                        CH, GE , GE );
        TANZEN.setSpezialisierungen(
                "Talent.Spezialisierung.Ausdruckstaenze",
                "Talent.Spezialisierung.Formationstaenze",
                "Talent.Spezialisierung.HoefischeTaenze",
                "Talent.Spezialisierung.Kulttaenze",
                "Talent.Spezialisierung.Meditationstaenze" );

        initialisieren( TASCHENDIEBSTAHL, SKT.D, MU, IN, FF );
        TASCHENDIEBSTAHL.einfuegen(
                Voraussetzung.MENSCHENKENNTNIS_4_STEIGERN );

        initialisieren( ZECHEN, SKT.D, IN, KO, KK );

        /*
         * GESELLSCHAFTLICHE TALENTE
         */
        initialisieren( BETOEREN, SKT.B, IN, CH, CH );
        BETOEREN.einfuegen( Voraussetzung.MENSCHENKENNTNIS_4 );

        initialisieren( ETIKETTE, SKT.B, KL, IN, CH );
        ETIKETTE.setSpezialisierungen(
                "Talent.Spezialisierung.Kulturkunde.Almada",
                "Talent.Spezialisierung.Kulturkunde.Amazonen",
                "Talent.Spezialisierung.Kulturkunde.Ambosszwerge",
                "Talent.Spezialisierung.Kulturkunde.AndergastNostria",
                "Talent.Spezialisierung.Kulturkunde.Aranien",
                "Talent.Spezialisierung.Kulturkunde.ArchaischeAchaz",
                "Talent.Spezialisierung.Kulturkunde.Auelfen",
                "Talent.Spezialisierung.Kulturkunde.Bornland",
                "Talent.Spezialisierung.Kulturkunde.Brilliantzwerge",
                "Talent.Spezialisierung.Kulturkunde.Bukanier",
                "Talent.Spezialisierung.Kulturkunde.Erzzwerge",
                "Talent.Spezialisierung.Kulturkunde.Ferkina",
                "Talent.Spezialisierung.Kulturkunde.Firnelfen",
                "Talent.Spezialisierung.Kulturkunde.Fjarninger",
                "Talent.Spezialisierung.Kulturkunde.Gjalskerlaender",
                "Talent.Spezialisierung.Kulturkunde.Goblins",
                "Talent.Spezialisierung.Kulturkunde.Grolme",
                "Talent.Spezialisierung.Kulturkunde.Horasreich",
                "Talent.Spezialisierung.Kulturkunde.Huegelzwerge",
                "Talent.Spezialisierung.Kulturkunde.Nivesen",
                "Talent.Spezialisierung.Kulturkunde.Norbarden",
                "Talent.Spezialisierung.Kulturkunde.NordaventurischeStaedte",
                "Talent.Spezialisierung.Kulturkunde.Maraskan",
                "Talent.Spezialisierung.Kulturkunde.Mohas",
                "Talent.Spezialisierung.Kulturkunde.Mittelreich",
                "Talent.Spezialisierung.Kulturkunde.Novadi",
                "Talent.Spezialisierung.Kulturkunde.Orks",
                "Talent.Spezialisierung.Kulturkunde.SchwarzeLande",
                "Talent.Spezialisierung.Kulturkunde.StammesAchaz",
                "Talent.Spezialisierung.Kulturkunde.Steppenelfen",
                "Talent.Spezialisierung.Kulturkunde.Suedaventurien",
                "Talent.Spezialisierung.Kulturkunde.Svellttal",
                "Talent.Spezialisierung.Kulturkunde.Thorwal",
                "Talent.Spezialisierung.Kulturkunde.Tocamuyac",
                "Talent.Spezialisierung.Kulturkunde.Trolle",
                "Talent.Spezialisierung.Kulturkunde.Trollzacker",
                "Talent.Spezialisierung.Kulturkunde.Tulamidenlande",
                "Talent.Spezialisierung.Kulturkunde.Waldelfen",
                "Talent.Spezialisierung.Kulturkunde.Zahori",
                "Talent.Spezialisierung.Kulturkunde.Zyklopeninseln" );

        initialisieren( GASSENWISSEN, SKT.B, KL, IN, CH );
        GASSENWISSEN.setSpezialisierungen(
                "Talent.Spezialisierung.Beschatten",
                "Talent.Spezialisierung.Hehlerei",
                "Talent.Spezialisierung.Kontakte",
                "Talent.Spezialisierung.Ortseinschaetzung" );

        initialisieren( LEHREN, SKT.B, KL, IN, CH );
        LEHREN.einfuegen( Voraussetzung.MENSCHENKENNTNIS_4_STEIGERN );
        LEHREN.setSpezialisierungen(
                "Talentgruppe.Kampf.Name",
                "Talentgruppe.Koerper.Name",
                "Talentgruppe.Gesellschaft.Name",
                "Talentgruppe.Natur.Name",
                "Talentgruppe.Wissen.Name",
                "Talentgruppe.Handwerk.Name",
                "Talent.Spezialisierung.SprachenSchriften" );

        initialisieren( MENSCHENKENNTNIS, SKT.B, KL, IN, CH );
        MENSCHENKENNTNIS.setSpezialisierungen(
                "Talent.Spezialisierung.Kulturkunde.Almada",
                "Talent.Spezialisierung.Kulturkunde.Amazonen",
                "Talent.Spezialisierung.Kulturkunde.Ambosszwerge",
                "Talent.Spezialisierung.Kulturkunde.AndergastNostria",
                "Talent.Spezialisierung.Kulturkunde.Aranien",
                "Talent.Spezialisierung.Kulturkunde.ArchaischeAchaz",
                "Talent.Spezialisierung.Kulturkunde.Auelfen",
                "Talent.Spezialisierung.Kulturkunde.Bornland",
                "Talent.Spezialisierung.Kulturkunde.Brilliantzwerge",
                "Talent.Spezialisierung.Kulturkunde.Bukanier",
                "Talent.Spezialisierung.Kulturkunde.Erzzwerge",
                "Talent.Spezialisierung.Kulturkunde.Ferkina",
                "Talent.Spezialisierung.Kulturkunde.Firnelfen",
                "Talent.Spezialisierung.Kulturkunde.Fjarninger",
                "Talent.Spezialisierung.Kulturkunde.Gjalskerlaender",
                "Talent.Spezialisierung.Kulturkunde.Goblins",
                "Talent.Spezialisierung.Kulturkunde.Grolme",
                "Talent.Spezialisierung.Kulturkunde.Horasreich",
                "Talent.Spezialisierung.Kulturkunde.Huegelzwerge",
                "Talent.Spezialisierung.Kulturkunde.Nivesen",
                "Talent.Spezialisierung.Kulturkunde.Norbarden",
                "Talent.Spezialisierung.Kulturkunde.NordaventurischeStaedte",
                "Talent.Spezialisierung.Kulturkunde.Maraskan",
                "Talent.Spezialisierung.Kulturkunde.Mohas",
                "Talent.Spezialisierung.Kulturkunde.Mittelreich",
                "Talent.Spezialisierung.Kulturkunde.Novadi",
                "Talent.Spezialisierung.Kulturkunde.Orks",
                "Talent.Spezialisierung.Kulturkunde.SchwarzeLande",
                "Talent.Spezialisierung.Kulturkunde.StammesAchaz",
                "Talent.Spezialisierung.Kulturkunde.Steppenelfen",
                "Talent.Spezialisierung.Kulturkunde.Suedaventurien",
                "Talent.Spezialisierung.Kulturkunde.Svellttal",
                "Talent.Spezialisierung.Kulturkunde.Thorwal",
                "Talent.Spezialisierung.Kulturkunde.Tocamuyac",
                "Talent.Spezialisierung.Kulturkunde.Trolle",
                "Talent.Spezialisierung.Kulturkunde.Trollzacker",
                "Talent.Spezialisierung.Kulturkunde.Tulamidenlande",
                "Talent.Spezialisierung.Kulturkunde.Waldelfen",
                "Talent.Spezialisierung.Kulturkunde.Zahori",
                "Talent.Spezialisierung.Kulturkunde.Zyklopeninseln" );

        initialisieren( SICH_VERKLEIDEN, SKT.B, MU, CH, GE );
        SICH_VERKLEIDEN.setSpezialisierungen(
                "Talent.Spezialisierung.AndererStand",
                "Talent.Spezialisierung.AnderesGeschlecht",
                "Talent.Spezialisierung.AndereRasse" );

        initialisieren( UEBERREDEN, SKT.B, MU, IN, CH );
        UEBERREDEN.einfuegen( Voraussetzung.MENSCHENKENNTNIS_4_STEIGERN );
        UEBERREDEN.setSpezialisierungen(
                "Talent.Spezialisierung.Aufwiegeln",
                "Talent.Spezialisierung.Betteln",
                "Talent.Spezialisierung.Einschuechtern",
                "Talent.Spezialisierung.Feilschen",
                "Talent.Spezialisierung.Luegen" );

        initialisieren( UEBERZEUGEN, SKT.B, KL, IN, CH );
        UEBERZEUGEN.setSpezialisierungen(
                "Talent.Spezialisierung.DemagogiePlaedoyerPredigt",
                "Talent.Spezialisierung.EinzelgespraechVerhoer",
                "Talent.Spezialisierung.DiskussionsRethorik",
                "Talent.Spezialisierung.SchriftlicheRethorik" );

        /*
         * NATUR-TALENTE
         */
        initialisieren( FAEHRTENSUCHEN, SKT.B, KL, IN, KO, IN );
        FAEHRTENSUCHEN.einfuegen( Voraussetzung.SINNENSCHAERFE_4_STEIGERN );
        FAEHRTENSUCHEN.setSpezialisierungen(
                "Talent.Spezialisierung.EisSchnee",
                "Talent.Spezialisierung.Gebirge",
                "Talent.Spezialisierung.Grasland",
                "Talent.Spezialisierung.Stadt",
                "Talent.Spezialisierung.Sumpf",
                "Talent.Spezialisierung.Unterirdisch",
                "Talent.Spezialisierung.WaldDschungel",
                "Talent.Spezialisierung.Wueste" );

        initialisieren( FALLENSTELLEN, SKT.B, KL, FF, KK );
        FALLENSTELLEN.einfuegen( Voraussetzung.WILDNISLEBEN_4_STEIGERN );
        FALLENSTELLEN.setSpezialisierungen(
                "Talent.Spezialisierung.Gruben",
                "Talent.Spezialisierung.Schlingen",
                "Talent.Spezialisierung.Speere" );

        initialisieren( FESSELN_ENTFESSELN,
                        Kosten.SCHLANGENMENSCH,
                        SKT.B,
                        Stufen.TALENT,
                        FF, GE, KK );
        FESSELN_ENTFESSELN.setSpezialisierungen(
                "Talent.Spezialisierung.Entfesseln",
                "Talent.Spezialisierung.Fesseln",
                "Talent.Spezialisierung.Knotenkunde",
                "Talent.Spezialisierung.TaueSpleissen",
                "Talent.Spezialisierung.NetzeKnuepfen" );

        initialisieren( FISCHEN_ANGELN, SKT.B, IN, FF, KK );
        FISCHEN_ANGELN.setSpezialisierungen(
                "Talent.Spezialisierung.FluesseBaeche",
                "Talent.Spezialisierung.Hochseeangeln",
                "Talent.Spezialisierung.SeenSumpfBrackwasser",
                "Talent.Spezialisierung.StrandRiff" );

        initialisieren( ORIENTIERUNG, SKT.B, KL, IN, IN );
        ORIENTIERUNG.setSpezialisierungen(
                "Talent.Spezialisierung.EisSchnee",
                "Talent.Spezialisierung.Dschungel",
                "Talent.Spezialisierung.Gebirge",
                "Talent.Spezialisierung.Grasland",
                "Talent.Spezialisierung.Meer",
                "Talent.Spezialisierung.Stadt",
                "Talent.Spezialisierung.Unterirdisch",
                "Talent.Spezialisierung.Wald",
                "Talent.Spezialisierung.Wueste" );

        initialisieren( WETTERVORHERSAGE, SKT.B, KL, IN, IN );

        initialisieren( WILDNISLEBEN, SKT.B, IN, GE, KO );
        WILDNISLEBEN.setSpezialisierungen(
                "Talent.Spezialisierung.Dschungel",
                "Talent.Spezialisierung.EisSchnee",
                "Talent.Spezialisierung.Gebirge",
                "Talent.Spezialisierung.Meer",
                "Talent.Spezialisierung.Steppe",
                "Talent.Spezialisierung.Sumpf",
                "Talent.Spezialisierung.Wald",
                "Talent.Spezialisierung.Wueste" );
        /*
         * WISSENS-TALENTE
         */
        initialisierenWissen( ANATOMIE, MU, KL, FF );
        ANATOMIE.einfuegen( Voraussetzung.NICHT_TOTENANGST );

        initialisierenWissen( BRETT_WUERFELSPIELE, KL, KL, IN );

        initialisierenWissen( GEOGRAPHIE, KL, KL, IN );

        initialisierenWissen( GESCHICHTSWISSEN, KL, KL, IN );
        GESCHICHTSWISSEN.setSpezialisierungen(
                "Talent.Spezialisierung.BauKunstgeschichte",
                "Talent.Spezialisierung.Militaergeschichte",
                "Talent.Spezialisierung.Religionsgeschichte",
                "Talent.Spezialisierung.Wissenschaftsgeschichte" );

        initialisierenWissen( GESTEINSKUNDE, KL, IN, FF );
        GESTEINSKUNDE.setSpezialisierungen(
                "Talent.Spezialisierung.Baugestein",
                "Talent.Spezialisierung.Edelmetalle",
                "Talent.Spezialisierung.Edelsteine" );

        initialisierenWissen( GOETTER_KULTE, KL, KL, IN );

        initialisierenWissen( HERALDIK, KL, KL, FF );

        initialisierenWissen( KRIEGSKUNST, MU, KL, CH );
        KRIEGSKUNST.setSpezialisierungen(
                "Talent.Spezialisierung.Logistik",
                "Talent.Spezialisierung.Militaergeschichte",
                "Talent.Spezialisierung.Monstren",
                "Talent.Spezialisierung.Seegefechte",
                "Talent.Spezialisierung.Strategie",
                "Talent.Spezialisierung.Taktik",
                "Talent.Spezialisierung.DrachenBekaempfung" );

        initialisierenWissen( MAGIEKUNDE, KL, KL, IN );
        MAGIEKUNDE.einfuegen( Voraussetzung.KULTURSPRACHE_6_STEIGERN );
        MAGIEKUNDE.setSpezialisierungen(
                "Talent.Spezialisierung.Artefaktherstellung",
                "Talent.Spezialisierung.Daemonologie",
                "Talent.Spezialisierung.Elementarismus",
                "Talent.Spezialisierung.Magiehistorie",
                "Talent.Spezialisierung.Magietheorie",
                "Talent.Spezialisierung.MagischeAnalyse",
                "Talent.Spezialisierung.Sphaerologie",
                "Talent.Spezialisierung.Zauberpraxis",
                "Talent.Spezialisierung.Zauberwerkstatt" );

        initialisierenWissen( MECHANIK, KL, KL, FF );
        MECHANIK.einfuegen( Voraussetzung.SCHRIFT_6_STEIGERN );
        MECHANIK.einfuegen( Voraussetzung.RECHNEN_6_STEIGERN );
        MECHANIK.einfuegen( Voraussetzung.MALEN_ZEICHNEN_6_STEIGERN );
        MECHANIK.setSpezialisierungen(
                "Talent.Spezialisierung.Belagerungswaffen",
                "Talent.Spezialisierung.KraeneHebewerkeMahlwerke",
                "Talent.Spezialisierung.Materialkunde",
                "Talent.Spezialisierung.PumpenWasserkraft",
                "Talent.Spezialisierung.Windkraft" );

        initialisierenWissen( PFLANZENKUNDE, KL, IN, FF );
        PFLANZENKUNDE.setSpezialisierungen(
                "Talent.Spezialisierung.Dschungel",
                "Talent.Spezialisierung.Gebirge",
                "Talent.Spezialisierung.Grasland",
                "Talent.Spezialisierung.Meer",
                "Talent.Spezialisierung.Steppe",
                "Talent.Spezialisierung.Sumpf",
                "Talent.Spezialisierung.Tundra",
                "Talent.Spezialisierung.Wald",
                "Talent.Spezialisierung.Wueste" );

        initialisierenWissen( RECHNEN, KL, KL, IN );
        RECHNEN.setSpezialisierungen(
                "Talent.Spezialisierung.Arithmetik",
                "Talent.Spezialisierung.Buchfuehrung",
                "Talent.Spezialisierung.Geometrie" );

        initialisierenWissen( RECHTSKUNDE, KL, KL, IN );
        RECHTSKUNDE.setSpezialisierungen(
                "Talent.Spezialisierung.Gildenrecht",
                "Talent.Spezialisierung.Kirchenrecht",
                "Talent.Spezialisierung.Strafrecht",
                "Talent.Spezialisierung.Staatsrecht" );

        initialisierenWissen( SAGEN_LEGENDEN, KL, IN, CH );
        SAGEN_LEGENDEN.setSpezialisierungen(
                "Talent.Spezialisierung.Kulturkunde.Almada",
                "Talent.Spezialisierung.Kulturkunde.Amazonen",
                "Talent.Spezialisierung.Kulturkunde.Ambosszwerge",
                "Talent.Spezialisierung.Kulturkunde.AndergastNostria",
                "Talent.Spezialisierung.Kulturkunde.Aranien",
                "Talent.Spezialisierung.Kulturkunde.ArchaischeAchaz",
                "Talent.Spezialisierung.Kulturkunde.Auelfen",
                "Talent.Spezialisierung.Kulturkunde.Bornland",
                "Talent.Spezialisierung.Kulturkunde.Brilliantzwerge",
                "Talent.Spezialisierung.Kulturkunde.Bukanier",
                "Talent.Spezialisierung.Kulturkunde.Erzzwerge",
                "Talent.Spezialisierung.Kulturkunde.Ferkina",
                "Talent.Spezialisierung.Kulturkunde.Firnelfen",
                "Talent.Spezialisierung.Kulturkunde.Fjarninger",
                "Talent.Spezialisierung.Kulturkunde.Gjalskerlaender",
                "Talent.Spezialisierung.Kulturkunde.Goblins",
                "Talent.Spezialisierung.Kulturkunde.Grolme",
                "Talent.Spezialisierung.Kulturkunde.Horasreich",
                "Talent.Spezialisierung.Kulturkunde.Huegelzwerge",
                "Talent.Spezialisierung.Kulturkunde.Nivesen",
                "Talent.Spezialisierung.Kulturkunde.Norbarden",
                "Talent.Spezialisierung.Kulturkunde.NordaventurischeStaedte",
                "Talent.Spezialisierung.Kulturkunde.Maraskan",
                "Talent.Spezialisierung.Kulturkunde.Mohas",
                "Talent.Spezialisierung.Kulturkunde.Mittelreich",
                "Talent.Spezialisierung.Kulturkunde.Novadi",
                "Talent.Spezialisierung.Kulturkunde.Orks",
                "Talent.Spezialisierung.Kulturkunde.SchwarzeLande",
                "Talent.Spezialisierung.Kulturkunde.StammesAchaz",
                "Talent.Spezialisierung.Kulturkunde.Steppenelfen",
                "Talent.Spezialisierung.Kulturkunde.Suedaventurien",
                "Talent.Spezialisierung.Kulturkunde.Svellttal",
                "Talent.Spezialisierung.Kulturkunde.Thorwal",
                "Talent.Spezialisierung.Kulturkunde.Tocamuyac",
                "Talent.Spezialisierung.Kulturkunde.Trolle",
                "Talent.Spezialisierung.Kulturkunde.Trollzacker",
                "Talent.Spezialisierung.Kulturkunde.Tulamidenlande",
                "Talent.Spezialisierung.Kulturkunde.Waldelfen",
                "Talent.Spezialisierung.Kulturkunde.Zahori",
                "Talent.Spezialisierung.Kulturkunde.Zyklopeninseln" );


        initialisierenWissen( SCHAETZEN, KL, IN, IN );
        SCHAETZEN.setSpezialisierungen(
                "Talent.Spezialisierung.Antiquitaeten",
                "Talent.Spezialisierung.Materialwert",
                "Talent.Spezialisierung.Schmuck" );

        initialisierenWissen( SPRACHENKUNDE, KL, KL, IN );

        initialisierenWissen( STERNKUNDE, KL, KL, IN );
        STERNKUNDE.einfuegen( Voraussetzung.GARETHI_TULAMIDYA_6_STEIGERN );
        STERNKUNDE.einfuegen( Voraussetzung.RECHNEN_6_STEIGERN );
        STERNKUNDE.einfuegen( Voraussetzung.SINNENSCHAERFE_6_STEIGERN );
        STERNKUNDE.setSpezialisierungen(
                "Talent.Spezialisierung.Himmelskartographie",
                "Talent.Spezialisierung.Horoskope",
                "Talent.Spezialisierung.Navigation",
                "Talent.Spezialisierung.Zeitbestimmung" );

        initialisierenWissen( STAATSKUNST, KL, IN, CH );
        STAATSKUNST.setSpezialisierungen(
                "Talent.Spezialisierung.Diplomatie",
                "Talent.Spezialisierung.Intrige",
                "Talent.Spezialisierung.Verwaltung" );

        initialisierenWissen( TIERKUNDE, MU, KL, IN );
        TIERKUNDE.setSpezialisierungen(
                "Talent.Spezialisierung.Drachen",
                "Talent.Spezialisierung.Jagdwild",
                "Talent.Spezialisierung.Meeresgetier",
                "Talent.Spezialisierung.Monstren",
                "Talent.Spezialisierung.Nutztiere",
                "Talent.Spezialisierung.Raubtiere",
                "Talent.Spezialisierung.Schaedlinge",
                "Talent.Spezialisierung.Vielbeiner",
                "Talent.Spezialisierung.Voegel" );

        /*
         * SPRACHEN
         */

        initialisiereSprache( ALAANI, SKT.A, 21, Sprachfamilie.TULAMIDYA,
                KUSLIKER_ZEICHEN );

        initialisiereSprache( ASDHARIA, SKT.B, 24, Sprachfamilie.ASDHARIA,
                ASDHARIA_SCHRIFT );

        initialisiereSprache( ATAK, SKT.A, 12, Sprachfamilie.ATAK );

        initialisiereSprache( BOSPARANO, SKT.A, 21, Sprachfamilie.GARETHI,
                KUSLIKER_ZEICHEN );

        initialisiereSprache(
                FUECHSISCH, SKT.A, 12, Sprachfamilie.FUECHSISCH );

        initialisiereSprache( GARETHI, SKT.A, 18, Sprachfamilie.GARETHI,
                KUSLIKER_ZEICHEN );

        initialisiereSprache(
                GOBLINISCH, SKT.A, 12, Sprachfamilie.GOBLINISCH );

        initialisiereSprache( ISDIRA, SKT.A, 21, Sprachfamilie.ISDIRA,
                ISDIRA_SCHRIFT );

        initialisiereSprache( MOHISCH, SKT.A, 15, Sprachfamilie.MOHISCH );

        initialisiereSprache( NUJUKA, SKT.A, 15, Sprachfamilie.NUJUKA );

        initialisiereSprache( OLOARKH, SKT.A, 10, Sprachfamilie.OLOARKH );

        initialisiereSprache(
                OLOGHAIJAN, SKT.A, 15, Sprachfamilie.OLOGHAIJAN );

        initialisiereSprache( ROGOLAN, SKT.A, 21, Sprachfamilie.ROGOLAN,
                ROGOLAN_SCHRIFT );

        initialisiereSprache( RSSAHH, SKT.A, 18, Sprachfamilie.RSSAHH,
                CHRMK );

        initialisiereSprache(
                THORWALSCH, SKT.A, 18, Sprachfamilie.THORWALSCH,
                KUSLIKER_ZEICHEN );

        initialisiereSprache( TULAMIDYA, SKT.A, 18, Sprachfamilie.TULAMIDYA,
                GLYPHEN_VON_UNAU, TULAMIDYA_SCHRIFT );

        initialisiereSprache(
                UR_TULAMIDYA, SKT.A, 21, Sprachfamilie.TULAMIDYA,
                UR_TULAMIDYA_SCHRIFT );

        initialisiereSprache( ZHAYAD, SKT.A, 15, Sprachfamilie.ZAHYAD,
                ZHAYAD_SCHRIFT );

        /*
         *  SCHRIFTEN
         */

        initialisiereSchrift( ASDHARIA_SCHRIFT, SKT.A, 18 );

        initialisiereSchrift( CHRMK, SKT.B, 18 );

        initialisiereSchrift( GLYPHEN_VON_UNAU, SKT.A, 13 );

        initialisiereSchrift( HJALDINGSCHE_RUNEN, SKT.A, 10 );

        initialisiereSchrift( ISDIRA_SCHRIFT, SKT.A, 15 );

        initialisiereSchrift( KUSLIKER_ZEICHEN, SKT.A, 10 );

        initialisiereSchrift( NANDURIA, SKT.A, 10 );

        initialisiereSchrift( ROGOLAN_SCHRIFT, SKT.A, 11 );

        initialisiereSchrift( TULAMIDYA_SCHRIFT, SKT.A, 14 );

        initialisiereSchrift( UR_TULAMIDYA_SCHRIFT, SKT.A, 16 );

        initialisiereSchrift( ZHAYAD_SCHRIFT, SKT.A, 18 );

        /*
         * HANDWERKS-TALENTE
         */
        initialisieren( ABRICHTEN, SKT.B, MU, IN, CH );
        ABRICHTEN.einfuegen( Voraussetzung.TIERKUNDE_4_STEIGERN );
        ABRICHTEN.setSpezialisierungen(
                "Talent.Spezialisierung.Dompteur",
                "Talent.Spezialisierung.Falkner",
                "Talent.Spezialisierung.Hundefuehrer",
                "Talent.Spezialisierung.Zureiter" );

        initialisieren( ACKERBAU, SKT.B, IN, FF, KO );

        initialisieren( ALCHIMIE, SKT.B, MU, KL, FF );
        ALCHIMIE.einfuegen( Voraussetzung.SCHRIFT_4 );
        ALCHIMIE.einfuegen( Voraussetzung.RECHNEN_4 );
        ALCHIMIE.setSpezialisierungen(
                "Talent.Spezialisierung.Brandmittel",
                "Talent.Spezialisierung.Gifte",
                "Talent.Spezialisierung.Laborpraxis",
                "Talent.Spezialisierung.ZauberElixiere",
                "Talent.Spezialisierung.FarbenLacke",
                "Talent.Spezialisierung.Materialveredelung" );

        initialisieren( BERGBAU, SKT.B, IN, KO, KK );
        BERGBAU.einfuegen( Voraussetzung.GESTEINSKUNDE_4 );
        BERGBAU.setSpezialisierungen(
                "Talent.Spezialisierung.Bergwerke",
                "Talent.Spezialisierung.Kanalisation",
                "Talent.Spezialisierung.NatuerlicheHoehlen",
                "Talent.Spezialisierung.Sappeur",
                "Talent.Spezialisierung.UnterirdischeStaedte" );

        initialisieren( BOGENBAU, SKT.B, KL, IN, FF );
        BOGENBAU.einfuegen( Voraussetzung.HOLZBEARBEITUNG_4 );
        BOGENBAU.setSpezialisierungen(
                "Talent.Spezialisierung.Armbrust",
                "Talent.Spezialisierung.Bogen",
                "Talent.Spezialisierung.BolzenPfeile",
                "Talent.Spezialisierung.Torsionswaffen" );

        initialisieren( BOOTE_FAHREN, SKT.B, GE, KO, KK );
        BOOTE_FAHREN.setSpezialisierungen(
                "Talent.Spezialisierung.Einmaster",
                "Talent.Spezialisierung.Floesse",
                "Talent.Spezialisierung.KanusKajaks",
                "Talent.Spezialisierung.Ruderboote" );

        initialisieren( FAHRZEUG_LENKEN, SKT.B, IN, CH, FF );
        FAHRZEUG_LENKEN.setSpezialisierungen(
                "Talent.Spezialisierung.Lastkarren",
                "Talent.Spezialisierung.Streitwagen",
                "Talent.Spezialisierung.Zuggespanne" );

        initialisieren( FALSCHSPIEL, SKT.B, MU, CH, FF );
        FALSCHSPIEL.einfuegen( Voraussetzung.MENSCHENKENNTNIS_4 );
        FALSCHSPIEL.setSpezialisierungen(
                "Talent.Spezialisierung.Wuerfel",
                "Talent.Spezialisierung.Karten" );

        initialisieren( FEINMECHANIK, SKT.B, KL, FF, FF );
        FEINMECHANIK.einfuegen( Voraussetzung.MALEN_ZEICHNEN_4 );
        FEINMECHANIK.setSpezialisierungen(
                "Talent.Spezialisierung.GoldSilberschmied",
                "Talent.Spezialisierung.Gravieren",
                "Talent.Spezialisierung.Schloesser",
                "Talent.Spezialisierung.Siegelstoecke",
                "Talent.Spezialisierung.TrickwaffenFallen",
                "Talent.Spezialisierung.Uhrwerke" );

        initialisieren( FLEISCHER, SKT.B, KL, FF, KK );
        FLEISCHER.setSpezialisierungen(
                "Talent.Spezialisierung.Fische",
                "Talent.Spezialisierung.Gefluegel",
                "Talent.Spezialisierung.Reptilien",
                "Talent.Spezialisierung.Saeugetiere" );

        initialisieren( GERBER_KUERSCHNER, SKT.B, KL, FF, KO );
        GERBER_KUERSCHNER.setSpezialisierungen(
                "Talent.Spezialisierung.Gerber",
                "Talent.Spezialisierung.Kuerschner",
                "Talent.Spezialisierung.Trophaeen" );

        initialisieren( GROBSCHMIED, SKT.B, FF, KO, KK );
        GROBSCHMIED.setSpezialisierungen(
                "Talent.Spezialisierung.Drahtzieher",
                "Talent.Spezialisierung.Hufschmied",
                "Talent.Spezialisierung.Plaettner",
                "Talent.Spezialisierung.Schwarzschmied",
                "Talent.Spezialisierung.Spengler",
                "Talent.Spezialisierung.Waffenschmied" );

        initialisieren( HEILKUNDE_GIFT, SKT.B, MU, KL, IN );
        HEILKUNDE_GIFT.setSpezialisierungen(
                "Talent.Spezialisierung.AlchimistischeZaubergifte",
                "Talent.Spezialisierung.MineralischeGifte",
                "Talent.Spezialisierung.PflanzlicheGifte",
                "Talent.Spezialisierung.TierischeGifte",
                "Talent.Spezialisierung.Atemgifte",
                "Talent.Spezialisierung.Einnahmegifte",
                "Talent.Spezialisierung.Kontaktgifte",
                "Talent.Spezialisierung.Waffengifte" );

        initialisieren( HEILKUNDE_KRANKHEITEN, SKT.B, MU, KL, CH );

        initialisieren( HEILKUNDE_SEELE, SKT.B, IN, CH, CH );

        initialisieren( HEILKUNDE_WUNDEN, SKT.B, KL, CH, FF );
        HEILKUNDE_WUNDEN.setSpezialisierungen(
                "Talent.Spezialisierung.BruecheQuetschungen",
                "Talent.Spezialisierung.Schnitte",
                "Talent.Spezialisierung.Verbrennungen",
                "Talent.Spezialisierung.Zahnleiden" );

        initialisieren( HOLZBEARBEITUNG, SKT.B, KL, FF, KK );
        HOLZBEARBEITUNG.setSpezialisierungen(
                "Talent.Spezialisierung.Beinschnitzerei",
                "Talent.Spezialisierung.Holzkunde",
                "Talent.Spezialisierung.Holzwaffen",
                "Talent.Spezialisierung.KueferWagner",
                "Talent.Spezialisierung.MoebelschreinereiTischlerei",
                "Talent.Spezialisierung.Schnitzen" );

        initialisieren( KARTOGRAPHIE, SKT.B, KL, KL, FF );
        KARTOGRAPHIE.einfuegen( Voraussetzung.MALEN_ZEICHNEN_4 );
        KARTOGRAPHIE.setSpezialisierungen(
                "Talent.Spezialisierung.Gebaeude",
                "Talent.Spezialisierung.HoehlenTunnel",
                "Talent.Spezialisierung.KuestenMeere",
                "Talent.Spezialisierung.Landschaften",
                "Talent.Spezialisierung.Stadt" );

        initialisieren( KOCHEN, SKT.B, KL, IN, FF );
        KOCHEN.setSpezialisierungen(
                "Talent.Spezialisierung.BackenBraten",
                "Talent.Spezialisierung.Festmahle",
                "Talent.Spezialisierung.Haltbarmachen",
                "Talent.Spezialisierung.Marschversorgung",
                "Talent.Spezialisierung.Traenke",
                "Talent.Spezialisierung.Vorkoster" );

        initialisieren( LEDERARBEITEN, SKT.B, KL, FF, FF );
        LEDERARBEITEN.setSpezialisierungen(
                "Talent.Spezialisierung.GurteRiemen",
                "Talent.Spezialisierung.Lederkleidung",
                "Talent.Spezialisierung.Lederruestungen",
                "Talent.Spezialisierung.Saettel",
                "Talent.Spezialisierung.Schuhwerk" );

        initialisieren( MALEN_ZEICHNEN, SKT.B, KL, IN, FF );
        MALEN_ZEICHNEN.setSpezialisierungen(
                "Talent.Spezialisierung.Architektur",
                "Talent.Spezialisierung.Landschaften",
                "Talent.Spezialisierung.Portraet",
                "Talent.Spezialisierung.TechnischeSkizze" );

        initialisieren( MUSIZIEREN, SKT.B, IN, CH, FF );
        MUSIZIEREN.setSpezialisierungen(
                "Talent.Spezialisierung.Blechblasinstrumente",
                "Talent.Spezialisierung.Floeten",
                "Talent.Spezialisierung.Harfen",
                "Talent.Spezialisierung.Lauten",
                "Talent.Spezialisierung.Rhythmusinstrumente",
                "Talent.Spezialisierung.Streichinstrumente",
                "Talent.Spezialisierung.Tasteninstrumente" );

        initialisieren( SCHLOESSER_KNACKEN, SKT.B, IN, FF, FF );
        SCHLOESSER_KNACKEN.setSpezialisierungen(
                "Talent.Spezialisierung.Fallen",
                "Talent.Spezialisierung.Schloesser" );

        initialisieren( SCHNEIDERN, SKT.B, KL, FF, FF );
        SCHNEIDERN.setSpezialisierungen(
                "Talent.Spezialisierung.EntwurfFestgewaender",
                "Talent.Spezialisierung.FlickGebrauchsschneiderei",
                "Talent.Spezialisierung.Tuchruestungen" );

        initialisieren( SEEFAHRT, SKT.B, FF, GE, KK );
        SEEFAHRT.setSpezialisierungen(
                "Talent.Spezialisierung.Ruderschiff",
                "Talent.Spezialisierung.Segelschiff" );

        initialisieren( STEINMETZ, SKT.B, FF, FF, KK );
        STEINMETZ.einfuegen( Voraussetzung.GESTEINSKUNDE_4 );
        STEINMETZ.setSpezialisierungen(
                "Talent.Spezialisierung.Baugestein",
                "Talent.Spezialisierung.Reliefs",
                "Talent.Spezialisierung.Statuen" );

        initialisieren( STEINSCHNEIDER, SKT.B, IN, FF, FF );
        STEINSCHNEIDER.einfuegen( Voraussetzung.GESTEINSKUNDE_4_STEIGERN );

        initialisieren( TAETOWIEREN, SKT.B, IN, FF, FF );
        TAETOWIEREN.einfuegen( Voraussetzung.MALEN_ZEICHNEN_4 );
        TAETOWIEREN.setSpezialisierungen(
                "Talent.Spezialisierung.NaturalistischeDarstellung",
                "Talent.Spezialisierung.Ornamente",
                "Talent.Spezialisierung.Zaubertaetowierungen" );

        initialisieren( ZIMMERMANN, SKT.B, KL, FF, KK );
        ZIMMERMANN.einfuegen( Voraussetzung.HOLZBEARBEITUNG_4_STEIGERN );
        ZIMMERMANN.setSpezialisierungen(
                "Talent.Spezialisierung.Dachdecker",
                "Talent.Spezialisierung.Holzkonstruktionen",
                "Talent.Spezialisierung.Schiffszimmermann" );

        /*
         * GABEN
         */
        initialisieren( GEFAHRENINSTINKT, SKT.F, KL, IN, IN );
        GEFAHRENINSTINKT.einfuegen( Voraussetzung.GEFAHRENINSTINKT );

        initialisieren( MAGIEGESPUER, SKT.F, MU, IN, IN );
        MAGIEGESPUER.einfuegen( Voraussetzung.MAGIEGESPUER );

        initialisieren( PROPHEZEIEN, SKT.F, IN, IN, CH );
        PROPHEZEIEN.einfuegen( Voraussetzung.PROPHEZEIEN );

        initialisieren( ZWERGENNASE, SKT.F, FF, IN, IN );
        ZWERGENNASE.einfuegen( Voraussetzung.ZWERGENNASE );

        /*
         * RITUALKENNTNIS
         */
        initialisieren( RITUALKENNTNIS, SKT.E, KL, null );
        RITUALKENNTNIS.einfuegen( Voraussetzung.NICHT_FREI_WAEHLBAR );
    }

    /** @return eine alphabetisch sortierte Liste aller Basis-Talente. */
    public static List< Talent > getBasisTalente() {
        
        final List< Talent > liste = 
            new ArrayList< Talent >( BASIS_TALENTE.values() );
        
        Collections.sort( liste, NAME_COMPARATOR );
        
        return liste;
    }

    /**
     * @return eine alphabetisch sortierte Liste aller Fernkampf-Techniken.
     *      Die Liste ist leer, so lange die Methode {@link #initialisieren()}
     *      nicht ausgefuehrt wurde.
     */
    public static List< Kampftechnik > getFernkampfTechniken() {
        
        final List< Kampftechnik > liste =
            new ArrayList< Kampftechnik >( FERNKAMPF_TALENTE.values() );
        
        Collections.sort( liste, NAME_COMPARATOR );
        
        return liste;
    }

    /**
     * @return eine alphabetisch sortierte Liste aller Talente, ausser den
     *      Gaben. Die Liste ist leer, so lange die Methode
     *      {@link #initialisieren()} nicht ausgefuehrt wurde.
     */
    public static List< Talent > getTalente() {
        
        final List< Talent > liste = 
            new ArrayList< Talent >( TALENTE_OHNE_GABEN.values() );
        
        Collections.sort( liste, NAME_COMPARATOR );
        
        return liste;
    }

    /**
     * Initialisiert alle Talente ausser Sprachen, Schriften, Kampftechniken und
     * Wissens-Talenten.
     *
     * @param talent
     *            das initialisiert werden soll.
     * @param steigerungsfaktor
     *            des Talents.
     * @param eigenschaft1
     * 			  die erste Probeneigenschaft
     * @param eigenschaft2
     * 			  die dritte Probeneigenschaft.
     * @param eigenschaften
     *            die dritte Probeneigenschaft oder eine Liste von
     *            Eigenschaften, wenn die Probe nicht immer gleich
     *            gewuerfelt wird.
     */
    private static void initialisieren(
            final Talent talent,
            final int steigerungsfaktor,
            final Eigenschaft eigenschaft1,
            final Eigenschaft eigenschaft2,
            final Eigenschaft ... eigenschaften ) {

        initialisieren(
                talent,
                Kosten.TALENT,
                steigerungsfaktor,
                Stufen.TALENT,
                eigenschaft1,
                eigenschaft2,
                eigenschaften );
    }

    /**
     * Initialisiert ein Talent mit den uebergebenen Werten. Diese Methode muss
     * nicht direkt aufgerufen werden, es gibt noch andere
     * initialisiere-Methoden die sich dieser hier bedienen. Jede andere
     * initialisieren Methode ruft diese hier auf.
     *
     * @param talent
     *            das initialisiert werden soll.
     * @param kosten
     *            fuer das Talent
     * @param steigerungsfaktor
     *            fuer das Talent.
     * @param stufen
     *            fuer das Talent.
     * @param eigenschaft1
     * 			  die erste Probeneigenschaft
     * @param eigenschaft2
     * 			  die zweite Probeneigenschaft
     * @param eigenschaften
     * 			  die dritte Probeneigenschaft oder eine Liste von
     * 			  Eigenschaften, wenn die Probe nicht immer gleich
     * 			  gewuerfelt wird.
     */
    private static void initialisieren(
            final Talent talent,
            final Kosten kosten,
            final int steigerungsfaktor,
            final Stufen stufen,
            final Eigenschaft eigenschaft1,
            final Eigenschaft eigenschaft2,
            final Eigenschaft ... eigenschaften  ) {

        talent.setKosten( kosten );
        talent.setSteigerungsspalte( steigerungsfaktor );
        talent.setStufen( stufen );
        talent.setProbe( eigenschaft1, eigenschaft2, eigenschaften );
        talent.einfuegen( Voraussetzung.STANDARD );

        assert ( ! TALENTE_MAP.containsKey( talent.getId() ) ) : 
            String.format(
                "Das Talent %1$s wurde zum zweiten mal in die Liste der " +
                "Talente eingefuegt.",
                talent.getName() );

        if ( ! talent.istGabe() ) {
            TALENTE_OHNE_GABEN.put( talent.getId(), talent );
        }

        TALENTE_MAP.put( talent.getId(), talent );
    }

    /**
     * Initialisiert die Wissens-Talente.
     */
    private static void initialisierenWissen(
            final Talent talent,
            final Eigenschaft eigenschaft1,
            final Eigenschaft eigenschaft2,
            final Eigenschaft ... eigenschaften ) {

        initialisieren(
                talent,
                Kosten.SCHRIFT_WISSEN,
                SKT.B,
                Stufen.TALENT,
                eigenschaft1,
                eigenschaft2,
                eigenschaften );
    }

    /**
     * Initialisiert eine Kampftechnik.
     *
     * @param kampftechnik
     *            die initialisiert werden soll.
     * @param steigerungsfaktor
     *            der Kampftechnik
     * @param eBe
     *            die Zahl, die fuer die effektive Behinderung von der
     *            Behinderung abgezogen wird.
     */
    private static void initialisiereKampftechnik(
            final Kampftechnik kampftechnik,
            final int steigerungsfaktor,
            final int eBe ) {

        kampftechnik.setEBe( eBe );

        if ( kampftechnik.istNahkampfTechnik() ) {

            if ( kampftechnik != LANZENREITEN ) {

                kampftechnik.einfuegen( Voraussetzung.KAMPFTECHNIK );
            }
        } else {

            FERNKAMPF_TALENTE.put( kampftechnik.getId(), kampftechnik );
        }

        if ( kampftechnik.istNahkampfTechnik() ) {

        	initialisieren(
        			kampftechnik,
        			Kosten.KAMPFTECHNIK,
        			steigerungsfaktor,
        			Stufen.TALENT,
        			GE, GE, KK );

        } else {

        	initialisieren(
                    kampftechnik,
                    Kosten.KAMPFTECHNIK,
                    steigerungsfaktor,
                    Stufen.TALENT,
                    GE, FF, KK );

        }
    }

    private static void initialisiereSchrift(
            final Schrift schrift,
            final int steigerungsfaktor,
            final int komplexitaet ) {

        initialisieren(
                schrift,
                Kosten.SCHRIFT_WISSEN,
                steigerungsfaktor,
                Stufen.SCHRIFT,
                KL, KL, FF );

        schrift.setKomplexitaet( komplexitaet );
    }
    /**
     * Initialisiert eine Sprache.
     *
     * @param sprache die initialisiert werden soll.
     * @param steigerungsfaktor der Sprache.
     * @param komplexitaet bis zu der die Sprache gesteigert werden darf.
     * @param sprachfamilie zu der die Sprache gehoert.
     * @param schriften in der die Sprache geschrieben wird.
     */
    private static void initialisiereSprache(
            final Sprache sprache,
            final int steigerungsfaktor,
            final int komplexitaet,
            final Sprachfamilie sprachfamilie,
            final Schrift ... schriften ) {

        initialisieren(
                sprache,
                Kosten.SPRACHE,
                steigerungsfaktor,
                Stufen.SPRACHE,
                KL, IN, CH );

        sprache.setKomplexitaet( komplexitaet );
        sprache.setSprachfamilie( sprachfamilie );
        for ( final Schrift schrift : schriften ) {
            sprache.einfuegen( schrift );
        }
    }

    /**
     * Prueft ob der uebergebene String die ID eines Talents ist.
     * 
     * @param string der ueberprueft werden soll. 
     * @return {@code true}, wenn es sich um eine Talent-ID handelt.
     */
    public static boolean istTalentId( final String string ) {
        
        return TALENTE_MAP.containsKey( string );
    }
    
    /**
     * Das Objekt, das sich um die Berechnung der Steigerungskosten des
     * Talents kuemmert.
     */
    private Kosten kosten;

    /** Die Probe die auf das Talent abgelegt wird. */
    private Eigenschaftsprobe probe;

    /** Die moeglichen Talentspezialisierungen. */
    private final List< Variante > spezialisierungen;

    /** Die Steigerungsspalte des Talents. */
    private int steigerungsspalte;

    /**
     * Ein Objekt, das sich um die Berechnung der Stufe und der
     * Maximal-/Minimal-Stufe kuemmert.
     */
    private Stufen stufen;

    /** Die Talengruppe zu der das Talent gehoert. */
    private final Talentgruppe talentgruppe;

    /**
     * Der Typ des Talents. Bestimmt ob es ein Basis-Talent, Spezial-Talent
     * oder eine Gabe ist.
     */
    private final Typ typ;

    /**
     * Die Voraussetzungen die fuer das Talent gelten.
     */
    private final Collection< Voraussetzung > voraussetzungen;

    /**
     * @param dieId des Talents.
     * @param dieTalentgruppe zu der das Talent gehoert.
     * @throws AssertionError wenn {@code derName} oder {@code dieTalentgruppe}
     *      {@code null} ist.
     */
    @SuppressWarnings("unchecked")
    Talent( final String dieId,
            final Talentgruppe dieTalentgruppe,
            final Typ derTyp ) {

        super( dieId );
        
        spezialisierungen = new ArrayList< Variante >();

        steigerungsspalte = -1;

        talentgruppe = dieTalentgruppe;

        talentgruppe.einfuegen( this );

        typ = derTyp;

        voraussetzungen = new ArrayList< Voraussetzung >();

        if ( Typ.BASIS == typ ) {

            BASIS_TALENTE.put( dieId, this );
        }
    }

    /**
     * Fuegt dem Talent eine weitere Voraussetzung hinzu.
     *
     * @param voraussetzung fuer das Talent.
     */
    final void einfuegen( final Voraussetzung voraussetzung ) {
        voraussetzungen.add( voraussetzung );
    }

    /**
     * @param id Die Id mit der die Bezeichnung der Spezialisierung aus der
     *          Lokalisierungsdatei geladen werden kann.
     */
    public final void einfuegenSpezialisierung( 
            final String id ) {

        final Variante spezialisierung =
            new Variante( id );
        
        if ( ! spezialisierungen.contains( spezialisierung ) ) {

            spezialisierungen.add( spezialisierung );
        }
    }

    /**
     * @param lokalisierungsschluessel
     *            der Talentspezialisierung die aus der Liste der 
     *            Spezialisierungen entfernt werden soll.
     */
    public final void entferneSpezialisierung( 
            final String lokalisierungsschluessel ) {

        spezialisierungen.remove(
                new Variante( lokalisierungsschluessel ) );
    }
    
    /**
     * @return die Regelelemente von denen das Talent in irgendeiner Weise
     *         abhaengig ist.
     */
    public final Collection< Object > getAbhaengigkeiten() {

        final Collection< Object > abhaengigkeiten = new HashSet< Object >();

        abhaengigkeiten.add( this );

        abhaengigkeiten.addAll( probe.getEigenschaften() );

        abhaengigkeiten.addAll( kosten.getAbhaengigkeiten() );

        abhaengigkeiten.addAll( stufen.getAbhaengigkeiten() );

        for ( final Voraussetzung voraussetzung : voraussetzungen ) {

            abhaengigkeiten.addAll( voraussetzung.getAbhaengigkeiten() );
        }

        return abhaengigkeiten;
    }

    /**
     * @param held
     *            der das Talent aktivieren will.
     * @return die AP die der Held zahlen muss um das Talent zu aktivieren.
     */
    public final int getAktivierungskosten( final Held held ) {
        return getAktivierungskosten( held, Lernmethode.KEINE );
    }

    /**
     * @param held
     *            der das Talent aktivieren will.
     * @param lernmethode
     *            die zum Aktivieren verwendet werden soll.
     * @return die AP die der Held zahlen muss um das Talent zu aktivieren.
     */
    public final int getAktivierungskosten(
            final Held held,
            final Lernmethode lernmethode ) {
        return kosten.getAktivierungskosten( held, this, lernmethode );
    }

    /**
     * @param held fuer den die Hoechsstufe berechnet werden soll.
     * @return die hoechste noch erlaubte Stufe fuer den Helden in diesem
     *      Talent.
     */
    public final int getHoechststufe( final Held held ) {
        return stufen.getHoechststufe( held, this );
    }

    /**
     * @return die Proben die auf diese Eigenschaft gewuerfelt werden.
     */
    public final Eigenschaftsprobe getProbe() {
        return probe;
    }

    /**
     * @return die Eigenschaften auf die bei einer Talentprobe gewuerfelt
     *      werden muss. Die Collection enthaelt jede Eigenschaft nur einmal,
     *      auch wenn mehrmals auf sie gewuerfelt wird.
     *      Im Fall der Kampftechniken, auf die keine Probe abgelegt wird,
     *      wird bei den Nahkampftechniken GE und KK geliefert, bei
     *      Fernkampftechniken FF, GE und KK.
     */
    public final Collection< Eigenschaft > getProbeneigenschaften() {

        return new HashSet< Eigenschaft >( probe.getEigenschaften() );
    }

    /**
     * @return die moeglichen Spezialisierungen des Talents fuer die
     *         Sonderfertigkeit Talentspezialisierung.
     */
    public final List< Variante > getSpezialisierungen() {
        
        Collections.sort( 
                spezialisierungen, 
                Variante.NAME_COMPARATOR );
        
        final List< Variante > liste =
            new ArrayList< Variante >();
        
        for ( Variante spezialisierung : spezialisierungen ) {
            
            if ( 0 == spezialisierung.getName().length() ) {
                
                continue;
            }
            
            liste.add( spezialisierung );
        }
        
        return liste;
    }

    /**
     * @see #getSteigerungsspalte(Held,int)
     * @return die Standardsteigerungsspalte des Talents. Die Steigerunsspalte
     *      der beim steigern eines Talentes gilt ist noch vom Helden und der
     *      Stufe des Talents abhaengig
     *      ({@link #getSteigerungsspalte(Held,int)}.
     */
    public final int getSteigerungsspalte() {
        return steigerungsspalte;
    }

    /**
     * @see #getSteigerungsspalte(Held,int)
     * @return die Standardsteigerungsspalte des Talents. Die Steigerunsspalte
     *      der beim steigern eines Talentes gilt ist noch vom Helden und der
     *      Stufe des Talents abhaengig
     *      ({@link #getSteigerungsspalte(Held,int)}.
     */
    public final int getSteigerungsspalte( Variante variante ) {
        return getSteigerungsspalte();
    }

    /**
     * Liefert die Steigerungsspalte fuer einen bestimmten Helden, in einem
     * Talent auf eine bestimmten Stufe gilt.
     *
     * @param held der das Talent steigern will.
     * @param stufe des Talents, das gesteigert werden soll.
     * @return die Steigerungsspalte des Talents fuer den Helden.
     */
    public final int getSteigerungsspalte( final Held held, final int stufe ) {

        return kosten.getSteigerungsspalte(
                held, this, stufe, Lernmethode.KEINE );
    }

    /**
     * Liefert die Steigerungsspalte fuer einen bestimmten Helden, in einem
     * Talent auf eine bestimmten Stufe gilt.
     *
     * @param held der das Talent steigern will.
     * @param stufe des Talents, das gesteigert werden soll.
     * @param lernmethode die zum Steigern verwendet werden soll.
     * @return den Steigerungsspalte des Talents fuer den Helden.
     */
    public final int getSteigerungsspalte(
            final Held held,
            final int stufe,
            final Lernmethode lernmethode ) {

        return kosten.getSteigerungsspalte( held, this, stufe, lernmethode );
    }

    /**
     * @return den Steigerungsfaktor des Talents.
     */
    public int getSteigerungsfaktor() {
        return SKT.getSteigerungsfaktor( steigerungsspalte );
    }

    /**
     * Liefert die AP-Kosten die zu bezahlen sind, um das Talent um einen Punkt,
     * zu steigern.
     *
     * @param held
     *            der das Talent steigern will.
     * @param stufe
     *            die das Talent aktuell besitzt.
     * @return die AP-Kosten um das Talent um einen Punkt zu steigern.
     */
    public final int getSteigerungskosten( final Held held, final int stufe ) {
        return getSteigerungskosten( held, stufe, Lernmethode.KEINE );
    }

    /**
     * Liefert die AP-Kosten die zu bezahlen sind, um das Talent um einen Punkt,
     * zu steigern.
     *
     * @param held
     *            der das Talent steigern will.
     * @param stufe
     *            die das Talent aktuell besitzt.
     * @param lernmethode
     *            die zum Steigern verwendet werden soll.
     * @return die AP-Kosten um das Talent um einen Punkt zu steigern.
     */
    public final int getSteigerungskosten(
            final Held held,
            final int stufe,
            final Lernmethode lernmethode ) {
        return kosten.getSteigerungskosten( held, this, stufe, lernmethode );
    }

    /**
     * Liefert die Modifikatin die ein Held auf ein Talent erhaelt.
     *
     * @param held der das Talent besitzt.
     * @return die Modifikation des Helden in dem Talent.
     */
    public final int getModifikation( final Held held ) {
        return stufen.getModifikation( held, this );
    }

    /** @return die Talentgruppe des Talents. */
    public final Talentgruppe getTalentgruppe() {
        return talentgruppe;
    }
    
    /**
     * @return {@code true} wenn es sich bei dem Talent um ein Basis-Talent
     *      handelt, {@code false} wenn es ein Spezial-Talent oder eine Gabe
     *      ist.
     */
    public final boolean istBasisTalent() {
        return Typ.BASIS == typ;
    }

    /**
     * @return {@code true} wenn es sich bei dem Talent um einr Gabe
     *      handelt, {@code false} wenn es ein normales Talent.
     */
    public final boolean istGabe() {
        return Typ.GABE == typ;
    }

    /**
     * Prueft ob der Held alle Voraussetzungen fuer dieses Talent einhaelt.
     *
     * @param held der das Talent waehlen will oder schon besitzt.
     * @return eine Beschreibung der aufgetretenen Probleme, wenn der Held
     *      nicht alle Voraussetzungen fuer das Talent erfuellt.
     */
    public final KonfliktBeschreibung pruefen( final Held held ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        for ( final Voraussetzung voraussetzung : voraussetzungen ) {
            beschreibung.einfuegen( voraussetzung.pruefen( held, this ) );
        }

        return beschreibung;
    }

    /**
     * Bestimmt die Kosten des Talents. Die Methode sollte nur einmal
     * aufgerufen werden um das Talent zu initialisieren.
     *
     * @param dieKosten Das Objekt das sich um die Berechnung der Kosten des
     *      Talents kuemmert.
     * @throws AssertionError wenn die Kosten schon gesetzt wurden.
     */
    private void setKosten( final Kosten dieKosten ) {

        assert( null == kosten ) : String.format(
                "Die Kosten von Talent %1$s sollten ein zweites mal gesetzt " +
                "werden.", getName() );

        kosten = dieKosten;
    }

    /**
     * Fuegt dem Talent eine weitere Eigenschaftsprobe hinzu.
     *
     * @param eigenschaft1 Die erste Eigenschaft auf die bei der Probe
     *      gewuerfelt wird.
     * @param eigenschaft2 Die zweite Eigenschaft auf die bei der Probe
     *      gewuerfelt wird.
     * @param eigenschaften Die dritte Eigenschaft auf die bei der Probe
     *      gewuerfelt wird, oder eine Reihe von Eigenschaften, wenn die
     *      Probe nicht immer gleich gewuerfelt wird.
     */
    private void setProbe(
            final Eigenschaft eigenschaft1,
            final Eigenschaft eigenschaft2,
            final Eigenschaft ... eigenschaften ) {

        probe = new Eigenschaftsprobe(
                eigenschaft1,
                eigenschaft2,
                eigenschaften );
    }

    /**
     * Bestimmt die moeglichen Spezialisierungen fuer die Sonderfertigkeit
     * Talentspezialisierung.
     *
     * @param ids
     *            die moeglichen Talentspezialisierungen.
     */
    final void setSpezialisierungen( final String ... ids ) {

        spezialisierungen.clear();

        for ( final String id : ids ) {

            spezialisierungen.add( new Variante( id ) );
        }
    }

    /**
     * Bestimmt den Standard-Steigerungsfaktor des Talents.
     * Die Methode sollte nur einmal aufgerufen werden um das Talent zu
     * initialisieren.
     *
     * @param derSteigerungsspalte fuer das Talent.
     * @throws AssertionError wenn der Steigerungsfaktor schon festgelegt
     *      wurde.
     */
    private void setSteigerungsspalte( final int derSteigerungsspalte ) {

        assert( -1 == steigerungsspalte ) : String.format(
                "Der Steigerungsfaktor von Talent %1$s sollte ein zweites " +
                "mal gesetzt werden.", getName() );

        steigerungsspalte = derSteigerungsspalte;
    }

    /**
     * Bestimmt das Objekt das sich um die Berechnung der Stufe des Talents
     * kuemmert.
     * Die Methode sollte nur einmal aufgerufen werden um das Talent zu
     * initialisieren.
     *
     * @param dieStufen Das Objekt das sich um die Berechnung der Stufe fuer
     *      das Talent kuemmert.
     * @throws AssertionError wenn die Stufe schon festgelegt wurde.
     */
    private void setStufen( final Stufen dieStufen ) {

        assert( null == stufen ) : String.format(
                "Das Stufen-Objekt von Talent %1$s sollte ein zweites mal " +
                "gesetzt werden.", getName() );

        stufen = dieStufen;
    }

    /**
     * @return {@link #getName()}
     */
    @Override
    public String toString() {
        return getName();
    }
}
