/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.talente.kosten;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.basis.SKT;
import de.bernhardjung.dsaprogramm.basis.varianten.Variante;
import de.bernhardjung.dsaprogramm.talente.Sprache;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

import java.util.Collection;

class KostenSprache extends Kosten {

    /**
     * Eine Begabung verbilligt die Steigerungskosten.
     * Andere Vorteile verbilligen sie ebenfalls.
     * Es gilt aber entweder die Verbilligung durch die Begabung oder
     * durch die Anderen Vorteile nicht beides zusammen.
     * Dies Variable ist true wenn die Verbilligung durch die Begabung
     * gelten soll.
     */
    private boolean begabungWirkt = false;

    private VorNachteilWert begabungTalent;

    private VorNachteilWert begabungSprachen;

    /** {@inheritDoc} */
    @Override
    public Collection<Object> getAbhaengigkeiten() {

        final Collection< Object > abhaengigkeiten = super.getAbhaengigkeiten();

        abhaengigkeiten.add( VorNachteil.AKADEMISCHE_AUSBILDUNG_GELEHRTER );

        abhaengigkeiten.add( VorNachteil.BEGABUNG_TALENT );

        abhaengigkeiten.add( VorNachteil.BEGABUNG_TALENTGRUPPE );

        abhaengigkeiten.add( VorNachteil.ELFISCHE_WELTSICHT );

        abhaengigkeiten.add( VorNachteil.GUTES_GEDAECHTNIS );

        return abhaengigkeiten;
    }

    /** {@inheritDoc} */
    @Override
    public int getSteigerungsspalte(
            final Held held,
            final Talent talent,
            final int stufe, Lernmethode lernmethode ) {

        int steigerungsfaktor = super.getSteigerungsspalte(held, talent, stufe, lernmethode);

        final Sprache sprache = (Sprache) talent;

        final Sprache muttersprache = held.getMuttersprache();

        final boolean istAusFremderSprachfamilie =
            muttersprache.getSprachfamilie() != sprache.getSprachfamilie();

        if ( !held.wirdErschaffen()
          && istAusFremderSprachfamilie
          && held.getZweitsprache() != sprache ) {

            steigerungsfaktor++;
        }

        setBegabung( talent );

        begabungWirkt = false;

        if ( ( held.hat( VorNachteil.GUTES_GEDAECHTNIS )
            || held.hat( VorNachteil.AKADEMISCHE_AUSBILDUNG_GELEHRTER ) )
          && ( held.hat( begabungTalent )
            || held.hat( begabungSprachen ) ) ) {
            // Der Held besitzt (GG + AAG) und eine Begabung.
            // Es wirken sich aber nur (GG und AAG) oder Begabung aus,
            // je nachdem was die groesste Verbilligung bringt.

            // Als erstes die Kosten durch GG und AAG feststellen.
            int faktorGGundAAG = steigerungsfaktor;

            if ( held.hat( VorNachteil.AKADEMISCHE_AUSBILDUNG_GELEHRTER )
                    && ( stufe < 10 )
                    && held.wirdErschaffen()
                    && istAusFremderSprachfamilie ) {

                --faktorGGundAAG;
            }

            float verbilligung = 1.0F;
            if ( held.hat( VorNachteil.GUTES_GEDAECHTNIS ) ) {

                verbilligung = 0.75F;
            }

            int kostenGGundAAG = SKT.getKosten( stufe+1, faktorGGundAAG );
            kostenGGundAAG = Math.round( verbilligung * kostenGGundAAG );

            // Jetzt die Kosten durch die Begabung ermitteln.
            int faktorBegabung = steigerungsfaktor;

            if ( held.hat( begabungTalent ) ) {
                --faktorBegabung;
            }

            if ( held.hat( begabungSprachen ) ) {
                --faktorBegabung;
            }

            final int kostenBegabung = SKT.getKosten( stufe+1, faktorBegabung );

            // Vergleichen welche genommen wird.
            if ( kostenBegabung < kostenGGundAAG ) {

                begabungWirkt = true;

                steigerungsfaktor = faktorBegabung;

            } else {

                steigerungsfaktor = faktorGGundAAG;

            }

        } else if ( held.hat( begabungTalent )
                 && held.hat( begabungSprachen ) ) {

            steigerungsfaktor -= 2;

        } else if ( held.hat( begabungTalent)
                 || held.hat( begabungSprachen )
                 || ( held.hat( VorNachteil.AKADEMISCHE_AUSBILDUNG_GELEHRTER )
                   && ( stufe < 10 )
                   && held.wirdErschaffen() ) ) {

            --steigerungsfaktor;
        }


        if ( held.hat( VorNachteil.ELFISCHE_WELTSICHT )
          && ( ! held.hatLeittalent( talent ) ) ) {

            ++steigerungsfaktor;
        }

        return steigerungsfaktor;
    }

    /** {@inheritDoc} */
    @Override
    public int getSteigerungskosten(
            final Held held,
            final Talent talent,
            final int stufe, Lernmethode lernmethode) {

        if ( lernmethode == Lernmethode.FREI )
            return 0;

        int kosten = super.getSteigerungskosten( held, talent, stufe, lernmethode );

        if ( ( ! begabungWirkt )
          && held.hat( VorNachteil.GUTES_GEDAECHTNIS ) ) {

            kosten = Math.round( 0.75F * kosten );
        }

        return kosten;
    }

    private void setBegabung( final Talent talent ) {

        if ( null == begabungTalent ) {

            begabungTalent = new VorNachteilWert( 
                    VorNachteil.BEGABUNG_TALENT,
                    new Variante( talent ) );

            begabungSprachen = new VorNachteilWert( 
                    VorNachteil.BEGABUNG_TALENTGRUPPE,
                    new Variante( Talentgruppe.SPRACHEN ) );

        } else {

            begabungTalent.setVariante( talent );

        }
    }
}
